/***************************************************************************
                          kbchartwindow.h  -  description
                             -------------------
    begin                : Thu Oct 31 2002
    copyright            : (C) 2002 by Richard Garand
    email                : richard@garandnet.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KBCHARTWINDOW_H
#define KBCHARTWINDOW_H

#include <config.h>

#include <qwidget.h>

#ifdef HAVE_KDCHART
#include <KDChartParams.h>
#include <KDChartTable.h>
#endif

#include <kmainwindow.h>

#include "account.h"
#include "kperiod.h"
#include "kbudgetdoc.h"

class KDChartWidget;
class KPeriodSelector;

/** A KBudget chart window. To use the window, set the necessary parameters
  * and then the type (it doesn't have to be in this order, but it's more
  * efficient). No special actions have to be taken when the data changes; the
  * window will connect to the necessary signals and update automatically.
  *@author Richard Garand
  */
class KBChartWindow : public KMainWindow {
   Q_OBJECT
  public:
    enum types { None = 0,
      /** Shows the total change in each account for the period. */
      ByAccount = 1,
      /** Shows the balance over time for cash accounts. */
      Balances = 2
      };
  
    KBChartWindow(KBudgetDoc *doc);
    ~KBChartWindow();

    static bool closeAllWindows();
    
    /** Sets the period to be displayed. */
    void setPeriod(KPeriod period);
    /** Sets the account to be displayed (where applicable); -1 means all accounts. */
    void setAccount(int account);
    /** Sets the account type to be displayed (where applicable). */
    void setAccountType(Account::types type);
    
    /** Sets the type of chart to show. */
    void setType(types type);

  protected:
    static QPtrList<KBChartWindow> instances;

  private:
    KBudgetDoc *m_doc;
    types m_type;
    KPeriod m_period;
    int m_account;
    Account::types m_acctType;

#ifdef HAVE_KDCHART
    KDChartParams m_params;
    KDChartTableData m_data;
#endif
    KDChartWidget *m_chartWidget;

    QStringList axisLabels;
    
    KPeriodSelector *periodSelector;
    
    void updateNone();
    void updateByAccount();
    void updateBalances();
    
    void (KBChartWindow::*update)();
    
  private slots:
    void slotPeriodChanged(KPeriod period);
    void slotAccountChanged(int id);
    void slotTransactionChanged(int id);
};

#endif
