/***************************************************************************
                          kbreconcilewindow.cpp  -  description
                             -------------------
    begin                : Mon Feb 3 2003
    copyright            : (C) 2003 by Richard Garand
    email                : richard@garandnet.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qvbox.h>
#include <kstdaction.h>
#include <kaction.h>
#include <klocale.h>
#include <kconfig.h>
#include <kglobal.h>
#include <kkeydialog.h>
#include <kedittoolbar.h>
#include <qpopupmenu.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qframe.h>
#include <knuminput.h>
 
#include <libkdepim/kdateedit.h>

#include "kbreconcilewindow.h"
#include "kbudgetdoc.h"
#include "ktransactionlist.h"
#include "knewtransaction.h"

QPtrList<KBReconcileWindow> KBReconcileWindow::instances;

KBReconcileWindow::KBReconcileWindow(KBudgetDoc* doc, int account)
  : KMainWindow(0,"reconcile_window"), m_finalBalance(0), m_statementBalance(0),
  m_doc(doc), m_account(account)
{
  // initialize window
  instances.append(this);
  setCaption(i18n("Reconcile account: %1").arg(doc->getAccount(account).name()));

  // create widgets
  QVBox* vb = new QVBox(this);
  setCentralWidget(vb);

  QHBox* hb = new QHBox(vb);
  vb->setStretchFactor(hb, 0);
  new QLabel(i18n("a date","From:"), hb);
  KDateEdit* fromDE = new KDateEdit(hb);
  new QLabel(i18n("a date","To:"), hb);
  KDateEdit* toDE = new KDateEdit(hb);

  new QLabel(i18n("Select the transactions that are not in your statement:"), vb);

  txnList = new KTransactionList(doc, vb, account, KTransactionList::Reconcile);
  vb->setStretchFactor(txnList, 10);

  QFrame* gridWidget = new QFrame(vb);
  vb->setStretchFactor(gridWidget, 0);
  QGridLayout* grid = new QGridLayout(gridWidget);

  QLabel* lb = new QLabel(i18n("Final balance:"), gridWidget);
  grid->addWidget(lb, 0, 0);
  finalBalance = new QLabel(gridWidget);
  finalBalance->setAlignment(Qt::AlignRight);
  grid->addWidget(finalBalance, 0, 1);

  lb = new QLabel(i18n("Statement balance:"), gridWidget);
  grid->addWidget(lb, 1, 0);
  statementBalance = new KDoubleNumInput(gridWidget);
  statementBalance->setRange(-9999999, 9999999, 0.01, false);
  grid->addWidget(statementBalance, 1, 1);

  lb = new QLabel(i18n("Difference:"), gridWidget);
  grid->addWidget(lb, 2, 0);
  diff = new QLabel(gridWidget);
  diff->setAlignment(Qt::AlignRight);
  grid->addWidget(diff, 2, 1);

  // initialize widgets; date range is all of the last month
  m_to = QDate::currentDate();
  m_to.setYMD(m_to.year(), m_to.month(), 1);
  m_from = m_to.addMonths(-1);
  m_to = m_to.addDays(-1);
  fromDE->setDate(m_from);
  toDE->setDate(m_to);

  // initialize actions
  (KStdAction::close(this, SLOT(close()), actionCollection()))->setStatusText(i18n("Closes the window"));
  transactionNew = new KAction(i18n("&New"), "mail_forward", Qt::Key_T, this, SLOT(slotNewTransaction()), actionCollection(), "transaction_new");
  transactionEdit = new KAction(i18n("&Edit"), "edit", Qt::Key_E, txnList, SLOT(slotEditTransaction()), actionCollection(), "transaction_edit");
  transactionDelete = new KAction(i18n("&Delete"), "editdelete", Qt::Key_D, txnList, SLOT(slotDeleteTransaction()), actionCollection(), "transaction_delete");
  showToolbar = KStdAction::showToolbar(this, SLOT(slotToggleToolBar()), actionCollection());
  KStdAction::keyBindings(this, SLOT(slotConfKeys()), actionCollection());
  KStdAction::configureToolbars(this, SLOT(slotConfToolbar()), actionCollection());

  // initialize GUI
  initGUI();

  // get popup menu
  mainPopup = static_cast<QPopupMenu*>(factory()->container("popup", this));
  if (!mainPopup) mainPopup = new QPopupMenu(this);

  // get configuration
  KConfig *conf = KGlobal::config();
  conf->setGroup("reconcileWindow_options");
  resize(400,300);    // using random numbers ripped from KNode :)
  applyMainWindowSettings(conf);
  showToolbar->setChecked(!toolBar()->isHidden());

  // connect signals
  connect(doc, SIGNAL(documentChanged()), SLOT(close()));
  connect(doc, SIGNAL(accountChanged(int, KBudgetDoc::ChangeAction)), SLOT(slotAccountChanged(int, KBudgetDoc::ChangeAction)));
  connect(fromDE, SIGNAL(dateChanged(QDate)), SLOT(slotFromDateChanged(QDate)));
  connect(toDE, SIGNAL(dateChanged(QDate)), SLOT(slotToDateChanged(QDate)));
  connect(txnList, SIGNAL(balanceChanged(float)), SLOT(slotFinalBalanceChanged(float)));
  connect(txnList, SIGNAL(contextMenuRequested(const QPoint&)), SLOT(slotContextMenuRequested(const QPoint&)));
  connect(statementBalance, SIGNAL(valueChanged(double)), SLOT(slotStatementBalanceChanged(double)));

  // finish initialization
  slotToDateChanged(m_to);
}

KBReconcileWindow::~KBReconcileWindow()
{
  instances.removeRef(this);
  KConfig *conf = KGlobal::config();
  conf->setGroup("reconcileWindow_options");
  saveMainWindowSettings(conf);
}

bool KBReconcileWindow::closeAllWindows()
{
  // TODO: limit this to the window that created this one
  QPtrList<KBReconcileWindow> list = instances;
  for (KBReconcileWindow* w = list.first(); w; w = list.next())
    w->close();
  return true;
}

void KBReconcileWindow::slotToggleToolBar()
{
  if(toolBar()->isVisible())
    toolBar()->hide();
  else
    toolBar()->show();
}

void KBReconcileWindow::slotConfKeys()
{
  KKeyDialog::configure(actionCollection());
}

void KBReconcileWindow::slotConfToolbar()
{
  KEditToolbar *dlg = new KEditToolbar(guiFactory(),this);
  if (dlg->exec())
    initGUI();
  delete dlg;
}

void KBReconcileWindow::slotNewTransaction()
{
  (new KNewTransaction(this, m_doc, m_account))->show();
}

void KBReconcileWindow::slotContextMenuRequested(const QPoint& p)
{
  mainPopup->popup(p);
}

void KBReconcileWindow::slotFromDateChanged(QDate from)
{
  setPeriod(from, m_to);
}

void KBReconcileWindow::slotToDateChanged(QDate to)
{
  setPeriod(m_from, to);
}

void KBReconcileWindow::slotFinalBalanceChanged(float to)
{
  m_finalBalance = to;
  finalBalance->setText(KGlobal::locale()->formatMoney(m_finalBalance));
  updateDifference();
}

void KBReconcileWindow::slotStatementBalanceChanged(double to)
{
  m_statementBalance = to;
  updateDifference();
}

void KBReconcileWindow::slotAccountChanged(int id, KBudgetDoc::ChangeAction action)
{
  if ( id == m_account && action == KBudgetDoc::Modified )
    setCaption(i18n("Reconcile account: %1").arg(m_doc->getAccount(m_account).name()));
}

void KBReconcileWindow::initGUI()
{
  createGUI("kbreconcilewindowui.rc");
}

void KBReconcileWindow::setPeriod(QDate from, QDate to)
{
  m_from = from;
  m_to = to;

  // make a period and pass it to the transaction widget
  txnList->slotPeriodChanged(KPeriod(from, to));
}

void KBReconcileWindow::updateDifference()
{
  float difference = m_finalBalance - m_statementBalance;
  diff->setText(KGlobal::locale()->formatMoney(difference));
}

#include "kbreconcilewindow.moc"
