/***************************************************************************
                          kbreconcilewindow.h  -  description
                             -------------------
    begin                : Mon Feb 3 2003
    copyright            : (C) 2003 by Richard Garand
    email                : richard@garandnet.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KBRECONCILEWINDOW_H
#define KBRECONCILEWINDOW_H

#include <qwidget.h>
#include <qdatetime.h>

#include <kmainwindow.h>

#include "kbudgetdoc.h"

class KAction;
class KToggleAction;
class QPopupMenu;
class QLabel;
class KDoubleNumInput;
class KTransactionList;

/**A window for reconciling an account
  *@author Richard Garand
  */

class KBReconcileWindow : public KMainWindow
{
  Q_OBJECT
  public: 
    KBReconcileWindow(KBudgetDoc* doc, int account);
    ~KBReconcileWindow();

    static bool closeAllWindows();

  public slots:
    void slotToggleToolBar();
    void slotConfKeys();
    void slotConfToolbar();
    void slotNewTransaction();

    void slotContextMenuRequested(const QPoint& p);
    
    void slotFromDateChanged(QDate from);
    void slotToDateChanged(QDate to);
    void slotFinalBalanceChanged(float to);
    void slotStatementBalanceChanged(double to);

    void slotAccountChanged(int id, KBudgetDoc::ChangeAction action);
    
  protected:
    static QPtrList<KBReconcileWindow> instances;
    
  private:
    void initGUI();
    /** sets the period to the given date range */
    void setPeriod(QDate from, QDate to);
    /** updates the difference */
    void updateDifference();

    // widgets
    KTransactionList* txnList;
    QLabel* finalBalance;
    KDoubleNumInput* statementBalance;
    QLabel* diff;

    // data
    QDate m_from;
    QDate m_to;
    float m_finalBalance;
    float m_statementBalance;

    // metadata
    KBudgetDoc* m_doc;
    int m_account;

    // actions
    KAction* fileClose;
    KAction* transactionNew;
    KAction* transactionEdit;
    KAction* transactionDelete;
    KToggleAction* showToolbar;

    QPopupMenu* mainPopup;
};

#endif
