/***************************************************************************
                          kbudget.cpp  -  description
                             -------------------
    begin                : Sat Aug 10 16:03:35 CST 2002
    copyright            : (C) 2002 by Richard Garand
    email                : richard@garandnet.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <config.h>
#include <stdlib.h>

// include files for QT
#include <qdir.h>
#include <qprinter.h>
#include <qpainter.h>

// include files for KDE
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <kstatusbar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kstdaction.h>
#include <kkeydialog.h>
#include <kedittoolbar.h>
#include <kdebug.h>

// application specific includes
#include "kbudget.h"
#include "kbudgetview.h"
#include "kbudgetdoc.h"
#include "ktransactionwindow.h"
#include "kbreconcilewindow.h"
#include "ktransactionsearchdlg.h"
#include "knewtransaction.h"

#ifdef HAVE_KDCHART
#include "kbchartwindow.h"
#endif

#define ID_STATUS_MSG 1

KBudgetApp::KBudgetApp(QWidget* , const char* name):KMainWindow(0, name)
{
  config=kapp->config();

  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initStatusBar();
  initActions();
  initPopups();
  initDocument();
  initView();

  readOptions();
  if ( firstTime ) {
    int result = KMessageBox::questionYesNo(0, i18n("Would you like to read a short tutorial? This tutorial is also availabe in the Getting Started section of the KBudget Handbook (in the Help menu)."), i18n("Tutorial"));
    if ( result == KMessageBox::Yes ) {
      kapp->invokeHelp( "getting-started" );
    }
  }
  
  ///////////////////////////////////////////////////////////////////
  // disable actions at startup
//  fileSave->setEnabled(false);
//  fileSaveAs->setEnabled(false);
  filePrint->setEnabled(false);
  editCut->setEnabled(false);
  editCopy->setEnabled(false);
  editPaste->setEnabled(false);
  viewYear->setEnabled(false);
  viewSpecial->setEnabled(false);
  accountsEdit->setEnabled(false);
  accountsTransactions->setEnabled(false);
  accountsReconcile->setEnabled(false);
  accountDelete->setEnabled(false);
  transactionsNew->setEnabled(false);
  slotUndoStatus(false, false);
}

KBudgetApp::~KBudgetApp()
{
  KTransactionWindow::closeAllWindows();
  KBReconcileWindow::closeAllWindows();
#ifdef HAVE_KDCHART
  KBChartWindow::closeAllWindows();
#endif
}

void KBudgetApp::initActions()
{
  fileNewWindow = new KAction(i18n("New &Window"), 0, 0, this, SLOT(slotFileNewWindow()), actionCollection(),"file_new_window");
  fileNew = KStdAction::openNew(this, SLOT(slotFileNew()), actionCollection());
  fileOpen = KStdAction::open(this, SLOT(slotFileOpen()), actionCollection());
  fileOpenRecent = KStdAction::openRecent(this, SLOT(slotFileOpenRecent(const KURL&)), actionCollection());
  fileSave = KStdAction::save(this, SLOT(slotFileSave()), actionCollection());
  fileSaveAs = KStdAction::saveAs(this, SLOT(slotFileSaveAs()), actionCollection());
  fileClose = KStdAction::close(this, SLOT(slotFileClose()), actionCollection());
  filePrint = KStdAction::print(this, SLOT(slotFilePrint()), actionCollection());
  fileQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());
  editUndo = KStdAction::undo(this, SLOT(slotEditUndo()), actionCollection());
  editRedo = KStdAction::redo(this, SLOT(slotEditRedo()), actionCollection());
  editCut = KStdAction::cut(this, SLOT(slotEditCut()), actionCollection());
  editCopy = KStdAction::copy(this, SLOT(slotEditCopy()), actionCollection());
  editPaste = KStdAction::paste(this, SLOT(slotEditPaste()), actionCollection());
  viewYear = new KAction(i18n("This &Year"), 0, Qt::Key_Y, this, SLOT(slotViewYear()), actionCollection(), "view_year");
  viewMonth = new KAction(i18n("This &Month"), 0, Qt::Key_M, this, SLOT(slotViewMonth()), actionCollection(), "view_month");
  viewSpecial = new KAction(i18n("Select &Period"), 0, 0, this, SLOT(slotViewSpecial()), actionCollection(), "select_period");
  accountsNew = new KAction(i18n("&New Account"), "folder_new", Qt::Key_A, 0, 0, actionCollection(), "accounts_new");
  accountsEdit = new KAction(i18n("&Edit Account"), "edit", Qt::Key_E, 0, 0, actionCollection(), "accounts_edit");
  accountDelete = new KAction(i18n("&Delete Account"), "editdelete", Qt::Key_D, this, SLOT(slotDeleteAccount()), actionCollection(), "accounts_delete");
  accountsTransactions = new KAction(i18n("&View Transactions"), "view_text", Qt::Key_V, 0, 0, actionCollection(), "accounts_transactions");
  accountsReconcile = new KAction(i18n("&Reconcile Account"), "kdict", Qt::Key_R, 0, 0, actionCollection(), "accounts_reconcile");
  transactionsNew = new KAction(i18n("&New Transaction"), "mail_forward", Qt::Key_T, 0, 0, actionCollection(), "transactions_new");
  transactionsView = new KAction(i18n("&View All Transactions"), "contents", Qt::CTRL+Qt::Key_T, 0, 0, actionCollection(), "transactions_view");
  transactionsFind = new KAction(i18n("&Find Transactions"), "find", Qt::CTRL+Qt::Key_F, this, SLOT(slotTransactionsFind()), actionCollection(), "transactions_find");
  viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection());
  viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection());
  KStdAction::keyBindings(this, SLOT(slotConfKeys()), actionCollection());
  KStdAction::configureToolbars(this, SLOT(slotConfToolbar()), actionCollection());

  fileNewWindow->setStatusText(i18n("Opens a new application window"));
  fileNew->setStatusText(i18n("Creates a new document"));
  fileOpen->setStatusText(i18n("Opens an existing document"));
  fileOpenRecent->setStatusText(i18n("Opens a recently used file"));
  fileSave->setStatusText(i18n("Saves the actual document"));
  fileSaveAs->setStatusText(i18n("Saves the actual document as..."));
  fileClose->setStatusText(i18n("Closes the actual document"));
  filePrint ->setStatusText(i18n("Prints out the actual document"));
  fileQuit->setStatusText(i18n("Quits the application"));
  editCut->setStatusText(i18n("Cuts the selected section and puts it to the clipboard"));
  editCopy->setStatusText(i18n("Copies the selected section to the clipboard"));
  editPaste->setStatusText(i18n("Pastes the clipboard contents to actual position"));
  viewYear->setStatusText(i18n("Opens the summary for the current year"));
  viewMonth->setStatusText(i18n("Opens the summary for the current month"));
  viewSpecial->setStatusText(i18n("Opens the summary for a selected period"));
  accountsNew->setStatusText(i18n("Creates a new acccount"));
  accountDelete->setStatusText(i18n("Deletes the currently selected account"));
  accountsTransactions->setStatusText(i18n("Opens a new window to display the account's transactions"));
  transactionsNew->setStatusText(i18n("Creates a new transaction"));
  transactionsView->setStatusText(i18n("Opens a new window to display this period's transactions"));
  transactionsFind->setStatusText(i18n("Searches transactions"));
  viewToolBar->setStatusText(i18n("Enables/disables the toolbar"));
  viewStatusBar->setStatusText(i18n("Enables/disables the statusbar"));

#ifdef HAVE_KDCHART
  reportsIncome = new KAction(i18n("&Income"), 0, 0, this, SLOT(slotReportsIncome()), actionCollection(), "income_chart");
  reportsSpending = new KAction(i18n("&Spending"), 0, 0, this, SLOT(slotReportsSpending()), actionCollection(), "spending_chart");
  reportsCash = new KAction(i18n("&Cashflow"), 0, 0, this, SLOT(slotReportsCash()), actionCollection(), "cash_chart");
  reportsBalance = new KAction(i18n("&Balance"), 0, 0, this, SLOT(slotReportsBalance()), actionCollection(), "balance_chart");
  
  reportsIncome->setStatusText(i18n("Displays a chart of income for the selected period"));
  reportsSpending->setStatusText(i18n("Displays a chart of spending for the selected period"));
  reportsCash->setStatusText(i18n("Displays a chart of cash flow for the selected period"));
  reportsBalance->setStatusText(i18n("Displays a chart of cash account balances"));
#endif

  // use the absolute path to your kbudgetui.rc file for testing purpose in createGUI();
  createGUI();

#if 0
  QPopupMenu *devPopup = new QPopupMenu(this);
  devPopup->insertItem("&Crash KBudget", this, SLOT(slotCrash()));
  menuBar()->insertItem("&Debugging", devPopup);
#endif
}


void KBudgetApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  // STATUSBAR
  // TODO: add your own items you need for displaying current application status.
  statusBar()->insertItem(i18n("Ready."), ID_STATUS_MSG);
}

void KBudgetApp::initDocument()
{
  doc = new KBudgetDoc(this);
  connect(doc, SIGNAL(saveableStatus(bool)), SLOT(slotEnableSave(bool)));
  connect(doc, SIGNAL(undoStatus(bool,bool)), SLOT(slotUndoStatus(bool,bool)));
  connect(doc, SIGNAL(accountsAvailable(bool)), transactionsNew, SLOT(setEnabled(bool)));
  connect(doc, SIGNAL(instantiateRecurrence(int)), SLOT(slotInstantiateRecurrence(int)));
  doc->newDocument();
}

void KBudgetApp::initView()
{ 
  ////////////////////////////////////////////////////////////////////
  // create the main widget here that is managed by KTMainWindow's view-region and
  // connect the widget to your document to display document contents.

  view = new KBudgetView(this);
  connect(view, SIGNAL(contextMenuRequested(const QPoint&)), SLOT(slotMainContextMenu(const QPoint&)));
  connect(accountsNew, SIGNAL(activated()), view, SLOT(slotNewAccount()));
  connect(accountsEdit, SIGNAL(activated()), view, SLOT(slotEditAccount()));
  connect(accountsTransactions, SIGNAL(activated()), view, SLOT(slotTransactions()));
  connect(accountsReconcile, SIGNAL(activated()), view, SLOT(slotReconcileAccount()));
  connect(transactionsNew, SIGNAL(activated()), view, SLOT(slotNewTransaction()));
  connect(transactionsView, SIGNAL(activated()), view, SLOT(slotAllTransactions()));

  connect(view, SIGNAL(accountSelectionStatus(bool)), accountsEdit, SLOT(setEnabled(bool)));
  connect(view, SIGNAL(accountSelectionStatus(bool)), accountsTransactions, SLOT(setEnabled(bool)));
  connect(view, SIGNAL(accountSelectionStatus(bool)), accountsReconcile, SLOT(setEnabled(bool)));
  connect(view, SIGNAL(accountSelectionStatus(bool)), accountDelete, SLOT(setEnabled(bool)));
  
  doc->addView(view);
  setCentralWidget(view);	
  setCaption(doc->URL().fileName(),false);

}

void KBudgetApp::initPopups()
{
  mainPopup = static_cast<QPopupMenu*>(factory()->container("account_view_popup", this));
  if (!mainPopup) mainPopup = new QPopupMenu(this);
}

void KBudgetApp::openDocumentFile(const KURL& url)
{
  slotStatusMsg(i18n("Opening file..."));

  doc->openDocument(url);
  fileOpenRecent->addURL(url);
  slotStatusMsg(i18n("Ready."));
}


KBudgetDoc *KBudgetApp::getDocument() const
{
  return doc;
}

void KBudgetApp::saveOptions()
{
  config->setGroup("General Options");
  config->writeEntry("Geometry", size());
  config->writeEntry("Show Toolbar", viewToolBar->isChecked());
  config->writeEntry("Show Statusbar",viewStatusBar->isChecked());
  config->writeEntry("ToolBarPos", (int) toolBar("mainToolBar")->barPos());
  fileOpenRecent->saveEntries(config);
}


void KBudgetApp::readOptions()
{
  config->setGroup("General Options");

  // bar status settings
  bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
  viewToolBar->setChecked(bViewToolbar);
  slotViewToolBar();

  bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
  viewStatusBar->setChecked(bViewStatusbar);
  slotViewStatusBar();


  // bar position settings
  KToolBar::BarPosition toolBarPos;
  toolBarPos=(KToolBar::BarPosition) config->readNumEntry("ToolBarPos", KToolBar::Top);
  toolBar("mainToolBar")->setBarPos(toolBarPos);

  // initialize the recent file list
  fileOpenRecent->loadEntries(config);

  QSize size=config->readSizeEntry("Geometry");
  if(!size.isEmpty())
  {
    resize(size);
  }

  firstTime = config->readBoolEntry("Tutorial prompt", true);
  config->writeEntry("Tutorial prompt", false);
}

void KBudgetApp::saveProperties(KConfig *_cfg)
{
  if(doc->URL().fileName()!=i18n("Untitled") && !doc->isModified())
  {
    // saving to tempfile not necessary

  }
  else
  {
    KURL url=doc->URL();	
    _cfg->writeEntry("filename", url.url());
    _cfg->writeEntry("modified", doc->isModified());
    QString tempname = kapp->tempSaveName(url.url());
    QString tempurl= KURL::encode_string(tempname);
    KURL _url(tempurl);
    doc->saveDocument(_url);
  }
}


void KBudgetApp::readProperties(KConfig* _cfg)
{
  QString filename = _cfg->readEntry("filename", "");
  KURL url(filename);
  bool modified = _cfg->readBoolEntry("modified", false);
  if(modified)
  {
    bool canRecover;
    QString tempname = kapp->checkRecoverFile(filename, canRecover);
    KURL _url(tempname);

    if(canRecover)
    {
      doc->openDocument(_url);
      doc->setModified();
      setCaption(_url.fileName(),true);
      QFile::remove(tempname);
    }
  }
  else
  {
    if(!filename.isEmpty())
    {
      doc->openDocument(url);
      setCaption(url.fileName(),false);
    }
  }
}

bool KBudgetApp::queryClose()
{
  return doc->saveModified();
}

bool KBudgetApp::queryExit()
{
  saveOptions();
  return true;
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KBudgetApp::slotFileNewWindow()
{
  slotStatusMsg(i18n("Opening a new application window..."));

  KBudgetApp *new_window= new KBudgetApp();
  new_window->show();

  slotStatusMsg(i18n("Ready."));
}

void KBudgetApp::slotFileNew()
{
  slotStatusMsg(i18n("Creating new document..."));

  if(!doc->saveModified())
  {
     // here saving wasn't successful

  }
  else
  {
    doc->newDocument();
    setCaption(doc->URL().fileName(), false);
  }

  slotStatusMsg(i18n("Ready."));
}

void KBudgetApp::slotFileOpen()
{
  slotStatusMsg(i18n("Opening file..."));

  if(!doc->saveModified())
  {
     // here saving wasn't successful

  }
  else
  {
    KURL url=KFileDialog::getOpenURL(QString::null,
        i18n("*.kbudget|KBudget files\n*|All files"), this, i18n("Open File..."));
    if(!url.isEmpty())
    {
      if ( doc->openDocument(url) ) {
        setCaption(url.fileName(), false);
        fileOpenRecent->addURL( url );
      } else {
        KMessageBox mb;
        mb.sorry(this, i18n("Error: couldn't open file"), i18n("Error"));
      }
    }
  }
  slotStatusMsg(i18n("Ready."));
}

void KBudgetApp::slotFileOpenRecent(const KURL& url)
{
  if ( url.isEmpty() )
    return;

  // TODO: fix fileOpenRecent bug
  slotStatusMsg(i18n("Opening file..."));

  if(!doc->saveModified())
  {
     // here saving wasn't successful
  }
  else
  {
    if ( doc->openDocument(url) )
      setCaption(url.fileName(), false);
    else {
      KMessageBox mb;
      mb.sorry(this, i18n("Error: couldn't open file"), i18n("Error"));
    }
  }

  slotStatusMsg(i18n("Ready."));
}

void KBudgetApp::slotFileSave()
{
  slotStatusMsg(i18n("Saving file..."));

  if ( doc->URL().filename() == i18n("Untitled") )
    slotFileSaveAs();
  else
    doc->saveDocument(doc->URL());

  slotStatusMsg(i18n("Ready."));
}

void KBudgetApp::slotFileSaveAs()
{
  slotStatusMsg(i18n("Saving file with a new filename..."));

  KURL url=KFileDialog::getSaveURL(QDir::currentDirPath(),
        i18n("*.kbudget|KBudget files\n*|All files"), this, i18n("Save as..."));
  if(!url.isEmpty())
  {
    if ( url.fileName().find('.') == -1 )
      url.setPath(url.path().append(".kbudget"));
    doc->saveDocument(url);
    fileOpenRecent->addURL(url);
    setCaption(url.fileName(),doc->isModified());
  }

  slotStatusMsg(i18n("Ready."));
}

void KBudgetApp::slotFileClose()
{
  slotStatusMsg(i18n("Closing file..."));

  close();

  slotStatusMsg(i18n("Ready."));
}

void KBudgetApp::slotFilePrint()
{
  slotStatusMsg(i18n("Printing..."));

  QPrinter printer;
  if (printer.setup(this))
  {
    view->print(&printer);
  }

  slotStatusMsg(i18n("Ready."));
}

void KBudgetApp::slotFileQuit()
{
  slotStatusMsg(i18n("Exiting..."));
  saveOptions();
  // close the first window, the list makes the next one the first again.
  // This ensures that queryClose() is called on each window to ask for closing
//  KMainWindow* w;
  if(memberList)
  {
    for ( unsigned int i= 0; i< memberList->count(); i++ )
    {
      // only close the window if the closeEvent is accepted. If the user presses Cancel on the saveModified() dialog,
      // the window and the application stay open.
      if ( memberList->at(i)->close() )
      break;
    }
  }
}

void KBudgetApp::slotEditUndo()
{
  doc->undo();
}

void KBudgetApp::slotEditRedo()
{
  doc->redo();
}

void KBudgetApp::slotEditCut()
{
  slotStatusMsg(i18n("Cutting selection..."));

  slotStatusMsg(i18n("Ready."));
}

void KBudgetApp::slotEditCopy()
{
  slotStatusMsg(i18n("Copying selection to clipboard..."));

  slotStatusMsg(i18n("Ready."));
}

void KBudgetApp::slotEditPaste()
{
  slotStatusMsg(i18n("Inserting clipboard contents..."));

  slotStatusMsg(i18n("Ready."));
}

void KBudgetApp::slotViewYear()
{
  slotStatusMsg(i18n("Peparing data..."));
  
  slotStatusMsg(i18n("Ready."));
}

void KBudgetApp::slotViewMonth()
{
  slotStatusMsg(i18n("Peparing data..."));

  QDate date = QDate::currentDate();
  view->selectMonth(date.year(), date.month());

  slotStatusMsg(i18n("Ready."));
}

void KBudgetApp::slotViewSpecial()
{
  slotStatusMsg(i18n("Peparing data..."));

  slotStatusMsg(i18n("Ready."));
}

void KBudgetApp::slotTransactionsFind()
{
  //(new KTransactionFindDlg(this, doc))->show();
}

void KBudgetApp::slotReportsIncome()
{
#ifdef HAVE_KDCHART
  slotStatusMsg(i18n("Creating income chart..."));
  KBChartWindow *w = new KBChartWindow(doc);
  w->setAccountType(Account::AC_INCOME);
  w->setPeriod(view->curPeriod());
  w->setType(KBChartWindow::ByAccount);
  w->show();
  slotStatusMsg(i18n("Ready."));
#endif
}

void KBudgetApp::slotReportsSpending()
{
#ifdef HAVE_KDCHART
  slotStatusMsg(i18n("Creating spending chart..."));
  KBChartWindow *w = new KBChartWindow(doc);
  w->setAccountType(Account::AC_EXPENSE);
  w->setPeriod(view->curPeriod());
  w->setType(KBChartWindow::ByAccount);
  w->show();
  slotStatusMsg(i18n("Ready."));
#endif
}

void KBudgetApp::slotReportsCash()
{
#ifdef HAVE_KDCHART
  slotStatusMsg(i18n("Creating cash chart..."));
  KBChartWindow *w = new KBChartWindow(doc);
  w->setAccountType(Account::AC_CASH);
  w->setPeriod(view->curPeriod());
  w->setType(KBChartWindow::ByAccount);
  w->show();
  slotStatusMsg(i18n("Ready."));
#endif
}

void KBudgetApp::slotReportsBalance()
{
#ifdef HAVE_KDCHART
  slotStatusMsg(i18n("Creating balance chart..."));
  KBChartWindow *w = new KBChartWindow(doc);
  w->setAccountType(Account::AC_CASH);
  w->setPeriod(view->curPeriod());
  w->setType(KBChartWindow::Balances);
  w->show();
  slotStatusMsg(i18n("Ready."));
#endif
}

void KBudgetApp::slotViewToolBar()
{
  slotStatusMsg(i18n("Toggling toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  if(!viewToolBar->isChecked())
    toolBar("mainToolBar")->hide();
  else
    toolBar("mainToolBar")->show();

  slotStatusMsg(i18n("Ready."));
}

void KBudgetApp::slotViewStatusBar()
{
  slotStatusMsg(i18n("Toggle the statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  if(!viewStatusBar->isChecked())
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  slotStatusMsg(i18n("Ready."));
}

void KBudgetApp::slotConfKeys()
{
  KKeyDialog::configure(actionCollection());
}

void KBudgetApp::slotConfToolbar()
{
  KEditToolbar *dlg = new KEditToolbar(guiFactory(),this);
  if (dlg->exec())
    createGUI();
  delete dlg;
}

void KBudgetApp::slotStatusMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG);
}

void KBudgetApp::slotEnableSave(bool enabled)
{
  fileSave->setEnabled(enabled);
}

void KBudgetApp::slotDeleteAccount()
{
  doc->deleteAccount(view->currentAccount());
}

void KBudgetApp::slotMainContextMenu(const QPoint &p)
{
  mainPopup->popup(p);
}

void KBudgetApp::slotUndoStatus(bool canUndo, bool canRedo)
{
  editUndo->setEnabled(canUndo);
  editRedo->setEnabled(canRedo);
}

void KBudgetApp::slotInstantiateRecurrence(int id)
{
  (new KNewRecurrence(this, doc, id))->show();
}

void KBudgetApp::slotCrash()
{
  kdDebug() << "slotCrash()\n";
  
  // do 1M random transaction operations
  for ( int i = 0; i < 1000000; i++ ) {
    if ( !(i % 10000) )
      printf("%i\n", i);
    int op = rand() % 3;
    switch (op) {
    case 0: // modify
      debugEditTransaction();
    case 1: // create
      debugCreateTransaction();
    case 2: // delete
      doc->deleteTransaction(debugGetRandomTransaction());
    }
  }
}

void KBudgetApp::debugCreateTransaction()
{
  int from, to;
  debugRandomAccts(from, to);
  doc->newTransaction(from, to, debugRandomValue(), debugRandomDate(), QString("this is only a test"));
}

void KBudgetApp::debugEditTransaction()
{
  Transaction &tr = doc->getTransaction(debugGetRandomTransaction());
  tr.setData(tr.from(), tr.to(), debugRandomValue(), debugRandomDate(), tr.description());
}

int KBudgetApp::debugGetRandomTransaction()
{
  QValueVector<Transaction*> trs = doc->getTransactions(KPeriod());
  return trs[rand() % trs.size()]->id();
}

QDate KBudgetApp::debugRandomDate()
{
  QDate date;
  // 2002-2004, any day
  date.setYMD(2002 + rand() % 3, 1 + (rand() % 12), 1);
  date = date.addDays(1 + (rand() % 31));
  return date;
}

void KBudgetApp::debugRandomAccts(int &from, int &to)
{
  QPtrList<Account> f, t;
  if ( rand() % 1 ) {
    // from income to cash
    f = doc->getAllAccounts(Account::AC_INCOME);
    t = doc->getAllAccounts(Account::AC_CASH);
  } else {
    // from cash to expense
    f = doc->getAllAccounts(Account::AC_CASH);
    t = doc->getAllAccounts(Account::AC_EXPENSE);
  }

  from = f.at(rand() % f.count())->id();
  to = t.at(rand() % t.count())->id();
}

float KBudgetApp::debugRandomValue()
{
  // return a random value up to $1M
  return (float)(rand() % 100000000) / 100;
}
