/***************************************************************************
                          kbudget.h  -  description
                             -------------------
    begin                : Sat Aug 10 16:03:35 CST 2002
    copyright            : (C) 2002 by Richard Garand
    email                : richard@garandnet.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KBUDGET_H
#define KBUDGET_H
 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// include files for Qt
#include <qdatetime.h>

// include files for KDE 
#include <kapp.h>
#include <kmainwindow.h>
#include <kaccel.h>
#include <kaction.h>
#include <kurl.h>

#include "knewaccount.h"

// forward declaration of the KBudget classes
class KBudgetDoc;
class KBudgetView;

/**
  * The base class for KBudget application windows. It sets up the main
  * window and reads the config file as well as providing a menubar, toolbar
  * and statusbar. An instance of KBudgetView creates your center view, which is connected
  * to the window's Doc object.
  * KBudgetApp reimplements the methods that KMainWindow provides for main window handling and supports
  * full session management as well as using KActions.
  * @see KMainWindow
  * @see KApplication
  * @see KConfig
  *
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
  * @version KDevelop version 1.2 code generation
  */
class KBudgetApp : public KMainWindow
{
  Q_OBJECT

  friend class KBudgetView;

  public:
    /** construtor of KBudgetApp, calls all init functions to create the application.
     */
    KBudgetApp(QWidget* parent=0, const char* name=0);
    ~KBudgetApp();
    /** opens a file specified by commandline option
     */
    void openDocumentFile(const KURL& url=0);
    /** returns a pointer to the current document connected to the KTMainWindow instance and is used by
     * the View class to access the document object's methods
     */	
    KBudgetDoc *getDocument() const; 	

  protected:
    /** save general Options like all bar positions and status as well as the geometry and the recent file list to the configuration
     * file
     */
    void saveOptions();
    /** read general Options again and initialize all variables like the recent file list
     */
    void readOptions();
    /** initializes the KActions of the application */
    void initActions();
    /** sets up the statusbar for the main window by initialzing a statuslabel.
     */
    void initStatusBar();
    /** initializes the document object of the main window that is connected to the view in initView().
     * @see initView();
     */
    void initDocument();
    /** creates the centerwidget of the KTMainWindow instance and sets it as the view
     */
    void initView();
    /** creates popup menus */
    void initPopups();
    /** queryClose is called by KTMainWindow on each closeEvent of a window. Against the
     * default implementation (only returns true), this calles saveModified() on the document object to ask if the document shall
     * be saved if Modified; on cancel the closeEvent is rejected.
     * @see KTMainWindow#queryClose
     * @see KTMainWindow#closeEvent
     */
    virtual bool queryClose();
    /** queryExit is called by KTMainWindow when the last window of the application is going to be closed during the closeEvent().
     * Against the default implementation that just returns true, this calls saveOptions() to save the settings of the last window's	
     * properties.
     * @see KTMainWindow#queryExit
     * @see KTMainWindow#closeEvent
     */
    virtual bool queryExit();
    /** saves the window properties for each open window during session end to the session config file, including saving the currently
     * opened file by a temporary filename provided by KApplication.
     * @see KTMainWindow#saveProperties
     */
    virtual void saveProperties(KConfig *_cfg);
    /** reads the session config file and restores the application's state including the last opened files and documents by reading the
     * temporary files saved by saveProperties()
     * @see KTMainWindow#readProperties
     */
    virtual void readProperties(KConfig *_cfg);

  public slots:
    /** open a new application window by creating a new instance of KBudgetApp */
    void slotFileNewWindow();
    /** clears the document in the actual view to reuse it as the new document */
    void slotFileNew();
    /** open a file and load it into the document*/
    void slotFileOpen();
    /** opens a file from the recent files menu */
    void slotFileOpenRecent(const KURL& url);
    /** save a document */
    void slotFileSave();
    /** save a document by a new filename*/
    void slotFileSaveAs();
    /** asks for saving if the file is modified, then closes the actual file and window*/
    void slotFileClose();
    /** print the actual file */
    void slotFilePrint();
    /** closes all open windows by calling close() on each memberList item until the list is empty, then quits the application.
     * If queryClose() returns false because the user canceled the saveModified() dialog, the closing breaks.
     */
    void slotFileQuit();
    void slotEditUndo();
    void slotEditRedo();
    /** put the marked text/object into the clipboard and remove
     *	it from the document
     */
    void slotEditCut();
    /** put the marked text/object into the clipboard
     */
    void slotEditCopy();
    /** paste the clipboard into the document
     */
    void slotEditPaste();
	/** view summary for the current year */
	void slotViewYear();
	/** view summary for the current month */
	void slotViewMonth();
	/** view summary for a special range */
	void slotViewSpecial();
	/** show the income report */
	void slotReportsIncome();
	/** show the spending report */
	void slotReportsSpending();
	/** show the cash report */
	void slotReportsCash();
    /** show the balance report */
    void slotReportsBalance();
    /** toggles the toolbar */
    void slotViewToolBar();
    void slotTransactionsFind();
    /** toggles the statusbar */
    void slotViewStatusBar();
    void slotConfKeys();
    void slotConfToolbar();
    /** changes the statusbar contents for the standard label permanently, used to indicate current actions.
     * @param text the text that is displayed in the statusbar
     */
    void slotStatusMsg(const QString &text);
    /** enables or disables the save actions */
    void slotEnableSave(bool enabled);
    /** deletes the currently selected account */
    void slotDeleteAccount();
    /** opens the main context menu */
    void slotMainContextMenu(const QPoint& p);
    void slotUndoStatus(bool canUndo, bool canRedo);
    /** Creates an instance of the next recurrence for the given transaction. */
    void slotInstantiateRecurrence(int id);
    
    /** Runs a stress test. */
    void slotCrash();
    /** Creates a random transaction. */
    void debugCreateTransaction();
    /** Randomizes the data for a random transaction. */
    void debugEditTransaction();
    /** Randomly selects a transaction and returns its ID. */
    int debugGetRandomTransaction();
    /** Creates a random date. */
    QDate debugRandomDate();
    /** Creates random from and to accounts. */
    void debugRandomAccts(int &from, int &to);
    /** Creates a random transaction value. */
    float debugRandomValue();

  private:
    bool firstTime;

    /** the configuration object of the application */
    KConfig *config;
    /** view is the main widget which represents your working area. The View
     * class should handle all events of the view widget.  It is kept empty so
     * you can create your view according to your application's needs by
     * changing the view class.
     */
    KBudgetView *view;
    /** doc represents your actual document and is created only once. It keeps
     * information such as filename and does the serialization of your files.
     */
    KBudgetDoc *doc;

    // KAction pointers to enable/disable actions
    KAction* fileNewWindow;
    KAction* fileNew;
    KAction* fileOpen;
    KRecentFilesAction* fileOpenRecent;
    KAction* fileSave;
    KAction* fileSaveAs;
    KAction* fileClose;
    KAction* filePrint;
    KAction* fileQuit;
    KAction* editUndo;
    KAction* editRedo;
    KAction* editCut;
    KAction* editCopy;
    KAction* editPaste;
    KAction* viewYear;
    KAction* viewMonth;
    KAction* viewSpecial;
    KAction* accountsNew;
    KAction* accountsEdit;
    KAction* accountsRename;
    KAction* accountsTransactions;
    KAction* accountsReconcile;
    KAction* transactionsNew;
    KAction* transactionsView;
    KAction* transactionsFind;
    KAction* reportsIncome;
    KAction* reportsSpending;
    KAction* reportsCash;
    KAction* reportsBalance;
    KToggleAction* viewToolBar;
    KToggleAction* viewStatusBar;
    KAction* accountDelete;

    QPopupMenu* mainPopup;
};
 
#endif // KBUDGET_H
