/***************************************************************************
                          kbudgetdoc.h  -  description
                             -------------------
    begin                : Sat Aug 10 16:03:35 CST 2002
    copyright            : (C) 2002 by Richard Garand
    email                : richard@garandnet.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KBUDGETDOC_H
#define KBUDGETDOC_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

#include <map>

// include files for QT
#include <qobject.h>
#include <qstring.h>
#include <qptrlist.h>
#include <qvaluevector.h>
#include <qmap.h>

// include files for KDE
#include <kurl.h>

#include "account.h"
#include "transaction.h"

// forward declaration of the KBudget classes
class KBudgetView;
class KBUndoAction;
class KBAccountUndo;
class KBUndoGroup;

/**	KBudgetDoc provides a document object for a document-view model.
  *
  * The KBudgetDoc class provides a document object that can be used in conjunction with the classes KBudgetApp and KBudgetView
  * to create a document-view model for standard KDE applications based on KApplication and KMainWindow. Thereby, the document object
  * is created by the KBudgetApp instance and contains the document structure with the according methods for manipulation of the document
  * data by KBudgetView objects. Also, KBudgetDoc contains the methods for serialization of the document data from and to files.
  *
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team. 	
  * @version KDevelop version 1.2 code generation
  */
class KBudgetDoc : public QObject
{
  Q_OBJECT
  public:
    friend class Account;
    friend class Transaction;
    friend class KBAccountUndo;
    friend class KBTransactionUndo;

    enum ChangeAction {Modified, Added, Deleted};

    /** Constructor for the fileclass of the application */
    KBudgetDoc(QWidget *parent, const char *name=0);
    /** Destructor for the fileclass of the application */
    ~KBudgetDoc();

    /** adds a view to the document which represents the document contents. Usually this is your main view. */
    void addView(KBudgetView *view);
    /** removes a view from the list of currently connected views */
    void removeView(KBudgetView *view);
    /** sets the modified flag for the document after a modifying action on the view connected to the document.*/
    void setModified(bool _m=true);
    /** returns if the document is modified or not. Use this to determine if your document needs saving by the user on closing.*/
    bool isModified(){ return modified; };
    /** "save modified" - asks the user for saving if the document is modified */
    bool saveModified();	
    /** deletes the document's contents */
    void deleteContents();
    /** initializes the document generally */
    bool newDocument();
    /** closes the acutal document */
    void closeDocument();
    /** loads the document by filename and format and emits the updateViews() signal */
    bool openDocument(const KURL& url, const char *format=0);
    /** saves the document under filename and format.*/	
    bool saveDocument(const KURL& url, const char *format=0);
    /** returns the KURL of the document */
    const KURL& URL() const;
    /** sets the URL of the document */
    void setURL(const KURL& url);

    /** return true if we have the account */
    bool haveAccount(const int id);
    /** return a reference the account with the requested ID.  */
    Account& getAccount(const int id);
    /** return a copy of the account with the requested ID containing data for the requested
      * month only. */
    Account getAccount(int id, QDate month);
    /** return list of accounts accessors for the given period */
    QValueVector<AccountAccessor> getAccounts(Account::types type, const KPeriod &period);
    /** retusn a list of all the accounts */
    QPtrList<Account> getAllAccounts(Account::types type = Account::AC_ALL);
    /** return the transaction with the requested ID */
    Transaction& getTransaction(const int id);
    /** returns a list of transactions in the given period, sorted by date in
      * ascending order */
    QValueVector<Transaction*> getTransactions(const KPeriod &period);

    /** create a new account, returns ID */
    int newAccount(Account::types type, QString name, float initialBalance);
    /** sets the name of an account */
    void setAccountName(int id, QString name);
    /** remove an account */
    void deleteAccount(int id);
    /** return true if the account id is valid */
    bool isValidAccount(int id);

    /** create a new transaction, returns ID */
    int newTransaction(int from, int to, float value, QDate date, QString description);
    /** update a transaction */
    void updateTransaction(int id, float value, QDate date, QString description);
    /** remove a transaction */
    void deleteTransaction(int id);

    void undo();
    void redo();
    /** Returns true if an undo action can be added; typically, this will only
      * return false if we are currently undoing or redoing something. */
    bool canAddUndo();
    /** Begins an undo group if there is no active group; all undo actions
      * between this and endUndoGroup() will be undone and redone together as
      * a single action. */
    void beginUndoGroup();
    /** Ends the current undo group, if any. */
    void endUndoGroup();

    /** suggests a from account for a new transaction given the currently
      * selected account */
    int suggestFrom(int account);
    /** suggests a to account for a new transaction given the currently
      * selected account */
    int suggestTo(int account);

  public slots:
    /** calls repaint() on all views connected to the document object and is called by the view by which the document has been changed.
     * As this view normally repaints itself, it is excluded from the paintEvent.
     */
    void slotUpdateAllViews(KBudgetView *sender);

  signals:
    /** a new document was created or loaded; throw away all old data */
    void documentChanged();
    /** A signal indicating that an account has changed; acct == -1 means
      * everything is changed. The account data will always be available
      * to the slots, even if the account is being deleted. */
    void accountChanged(int acct, KBudgetDoc::ChangeAction change);
    /** Emitted after an account has been deleted; account data will no longer
      * be available. */
    void accountDeleted(int acct);
    /** A signal indicating that a transaction has changed. The transaction data
      * will always be available to the slots, even if the transaction is being
      * deleted. */
    void transactionChanged(int tx, KBudgetDoc::ChangeAction change);
    /** Emitted after a transaction has been deleted; account data will no
      * longer be available. */
    void transactionDeleted(int tx);
    /** a signal indicating that the document modification status changed */
    void saveableStatus(bool status);
    /** enabled/disables undo and redo actions */
    void undoStatus(bool canUndo, bool canRedo);
    /** indicates if accounts exist */
    void accountsAvailable(bool);
    /** Signals that an existing recurring transaction should be duplicated at
      * a new date, most likely after asking the user. Only one slot should act
      * on this. All information necessary can be extracted from the
      * transaction. */
    void instantiateRecurrence(int id);

  public:
    /** the list of the views currently connected to the document */
    static QList<KBudgetView> *pViewList;

  private slots:
    /** Checks the next transaction that will recur; if the recurrence date is
      * <= today, emits instantiateRecurrence. */
    void slotCheckRecurrences();

  private:
    QString readFile(QString filename);
    void readFlatFile(QTextStream &inf, QString name);
    void signalAcctChanged(int id);
    void signalTransactionChanged(int id);
    void signalTransactionRecurrenceChanged(int id);
    /** write the document to the specified file */
    bool writeDocument(QString file);
    /** inserts an account with the id given */
    void insertAccount(int id, Account* copy);
    /** inserts a transaction with the id given */
    void insertTransaction(int id, Transaction* copy);
    /** adds an undo action */
    void addUndo(KBUndoAction* a);

    /** the modified flag of the current document */
    bool modified;
    KURL doc_url;
    QMap<int, Account> accounts[3];
    QMap<int, Transaction> transactions;
    /** next account ID to use */
    int nextAccount;
    /** next transaction ID to use */
    int nextTransaction;

    /** undo history */
    QPtrList<KBUndoAction> undos;
    /** undos.at() when the document was last saved */
    int lastSavedPos;
    /** true if we are in an undo/redo function */
    bool inUndo;
    KBUndoGroup* ugroup;

    /** in case getAccount fails */
    Account blankAcct;
    /** in case getTransaction fails */
    Transaction blankTransaction;
    
    /** List of recurrence transactions, sorted in ascending order by date of
      * next recurrence. */
    std::multimap<QDateTime,int> recurrences;
    /** Time for updating recurrences. */
    QTimer *rTimer;
};

#endif // KBUDGETDOC_H
