/***************************************************************************
                          kbudgeteditor.cpp  -  description
                             -------------------
    begin                : Sat Aug 31 2002
    copyright            : (C) 2002 by Richard Garand
    email                : richard@garandnet.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qframe.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qdatetime.h>
#include <qhbox.h>

#include <klocale.h>
#include <knuminput.h>
#include <kpushbutton.h>
#include <kdebug.h>

#include "kbudgeteditor.h"
#include "kbudgetdoc.h"

// just keeping some translatable strings around in case I decide to do multiple
// budget entries with recurrence
const char *dontuse[] = {I18N_NOOP("Remove"), I18N_NOOP("Recurrence...")};

/*KBudgetEntryEditor::KBudgetEntryEditor(KBudgetEditor *parent, QString text, bool editable)
  : QObject(parent), m_parent(parent), m_label(0), m_lineEdit(0), m_amtEditor(0),
  m_amtLabel(0)
{
  bool totalRow = false;
  // if there is only one row, add this at the end; otherwise, insert before the
  // last row
  row = m_parent->nextRow++;
  kdDebug() << "Inserting at row "<< row << endl;
  if ( row == 0 ) {
    row = 1;
    totalRow = true;
  } else
    m_parent->editors.getLast()->move(1);

  if ( editable ) {
    kdDebug() << "Added line edit at " << row << ", " << 0 << endl;
    m_lineEdit = new KLineEdit(m_parent);
    widgets.push_back(m_lineEdit);
  } else {
    kdDebug() << "Added label at " << row << ", " << 0 << endl;
    m_label = new QLabel(text, m_parent);
    widgets.push_back(m_label);
  }
  m_parent->layout->addWidget(widgets.back(), row, 0);
  if ( !totalRow ) {
    m_amtEditor = new KDoubleNumInput(m_parent);
    m_amtEditor->setPrecision(2);
    m_amtEditor->setRange(-9999999, 9999999, 0.01, false);
    widgets.push_back(m_amtEditor);
  } else {
    m_amtLabel = new QLabel(KGlobal::locale()->formatMoney(0), m_parent);
    m_amtLabel->setAlignment(Qt::AlignRight);
    widgets.push_back(m_amtLabel);
  }
  m_parent->layout->addWidget(widgets.back(), row, 1);

  // button rules: all except total and first item (second in list) have buttons
  if ( !totalRow && m_parent->editors.count() > 1 ) {
    KPushButton *button = new KPushButton(m_parent);
    button->setText(i18n("Remove"));
    m_parent->layout->addWidget(button, 1, 2);
    widgets.push_back(button);
    connect(button, SIGNAL(clicked()), SLOT(slotRemove()));

    button = new KPushButton(m_parent);
    button->setText(i18n("Recurrence..."));
    m_parent->layout->addWidget(button, 1, 3);
    widgets.push_back(button);
    connect(button, SIGNAL(clicked()), SLOT(slotRecurrence()));
  }
  
  // make sure the editors list mirrors the physical ordering
  m_parent->editors.insert(row - 1, this);
}

QString KBudgetEntryEditor::text() const
{
  if ( m_lineEdit )
    return m_lineEdit->text();
  else
    return m_label->text();
}

void KBudgetEntryEditor::setText(QString text)
{
  if ( m_lineEdit )
    m_lineEdit->setText(text);
  else
    m_label->setText(text);
}

float KBudgetEntryEditor::value() const
{
  if ( m_amtEditor )
    return m_amtEditor->value();
  return 0.0f;
}

void KBudgetEntryEditor::setValue(float value)
{
  if ( m_amtEditor )
    m_amtEditor->setValue(value);
  else
    m_amtLabel->setText(KGlobal::locale()->formatMoney(value));
}

void KBudgetEntryEditor::slotRemove()
{
  // TODO: confirm?

  for ( uint i = 0; i < widgets.size(); i++ )
    if ( widgets[i] )
      delete widgets[i];
  
  m_parent->nextRow--;

  m_parent->editors.find(this);
  for ( m_parent->editors.next(); m_parent->editors.current(); m_parent->editors.next() )
    m_parent->editors.current()->move(-1);
  m_parent->editors.remove(this);
  
  delete this;
}

void KBudgetEntryEditor::slotRecurrence()
{
}

void KBudgetEntryEditor::move(int rows)
{
  kdDebug() << "Moving from row " << row << " to row " << row + rows << endl;
  for ( uint i = 0; i < widgets.size(); i++ ) {
    if ( !widgets[i] )
      continue;
    m_parent->layout->remove(widgets[i]);
    m_parent->layout->addWidget(widgets[i], row + rows, i);
  }
  row += rows;
}*/

KBudgetEditor::KBudgetEditor(QWidget* parent, KBudgetDoc* doc)
  : QFrame(parent, "budget_editor"), m_doc(doc), account(-1), m_selecting(false)
{
  QGridLayout* grid = new QGridLayout(this);

  prevMonthLabel = new QLabel(i18n("Remaining from:"), this);
  grid->addWidget(prevMonthLabel, 0, 0);
  prevMonthValue = new QLabel(KGlobal::locale()->formatMoney(0), this);
  prevMonthValue->setAlignment(Qt::AlignRight);
  grid->addWidget(prevMonthValue, 0, 1);

  thisMonthLabel = new QLabel(i18n("Budgeted for:"), this);
  grid->addWidget(thisMonthLabel, 1, 0);
  thisMonthValue = new KDoubleNumInput(this, "thisMonthValue");
  thisMonthValue->setPrecision(2);
  thisMonthValue->setRange(-9999999, 9999999, 0.01, false);
  grid->addWidget(thisMonthValue, 1, 1);

  QLabel* totalLabel = new QLabel(i18n("Total:"), this);
  grid->addWidget(totalLabel, 2, 0);
  total = new QLabel(KGlobal::locale()->formatMoney(0), this);
  total->setAlignment(Qt::AlignRight);
  grid->addWidget(total, 2, 1);

  QSize gridSize = grid->sizeHint();
  setFixedHeight(gridSize.height());

  grid->setColStretch(1, 1);
  grid->setColStretch(2, 1);

  KPushButton* button = new KPushButton(this);
  button->setText("Close");
  grid->addMultiCellWidget(button, 0, 2, 3, 3);

  connect(button, SIGNAL(clicked()), SLOT(slotHide()));
  connect(thisMonthValue, SIGNAL(valueChanged(double)), SLOT(slotValueChanged(double)));
  connect(m_doc, SIGNAL(accountChanged(int,KBudgetDoc::ChangeAction)), SLOT(slotAccountChanged(int,KBudgetDoc::ChangeAction)));

  hide();
}

KBudgetEditor::~KBudgetEditor()
{
}

void KBudgetEditor::selectPeriod(const KPeriod &period)
{
  m_selecting = true;
  m_period = period;

  if ( period.type() == KPeriod::Month ) {
    m_period.previous();
    prevMonthLabel->setText(i18n("Remaining from %1:").arg(m_period.name()));
    m_period.next();
    thisMonthLabel->setText(i18n("Budgeted for %1:").arg(m_period.name()));
  }

  if ( account != -1 ) {
    selectAccount(account);
  } else {
    prevMonthValue->setText(KGlobal::locale()->formatMoney(0));
    thisMonthValue->setValue(0);
    total->setText(KGlobal::locale()->formatMoney(0));
  }

  m_selecting = false;
}

void KBudgetEditor::selectAccount(int id)
{
  m_selecting = true;

  account = id;
  if ( m_period.type() == KPeriod::Month ) {
    AccountAccessor acct(&m_doc->getAccount(account), m_period);
    float pmValue = acct.startingBudget();
    prevMonthValue->setText(KGlobal::locale()->formatMoney(pmValue));
    float cmValue = acct.budgeted();
    thisMonthValue->setValue(cmValue);
    total->setText(KGlobal::locale()->formatMoney(pmValue + cmValue));
  }

  m_selecting = false;
}

void KBudgetEditor::slotValueChanged(double value)
{
  if ( m_selecting )
    return;

  if ( account != -1 && m_period.type() == KPeriod::Month ) {
    m_doc->getAccount(account).setBudgeted(m_period.firstDay(), (float)value);
    selectAccount(account);
    emit valueChanged(account);
  } else
    thisMonthValue->setValue(0);
}

void KBudgetEditor::slotHide()
{
  hide();
}

void KBudgetEditor::slotAccountChanged(int acct, KBudgetDoc::ChangeAction action)
{
  if ( account != -1 && acct == account && action == KBudgetDoc::Modified ) {
    selectAccount(acct);
  }
}

#include "kbudgeteditor.moc"
