/***************************************************************************
                          kbudgeteditor.h  -  description
                             -------------------
    begin                : Sat Aug 31 2002
    copyright            : (C) 2002 by Richard Garand
    email                : richard@garandnet.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KBUDGETEDITOR_H
#define KBUDGETEDITOR_H

#include <qframe.h>

#include "account.h"
#include "kbudgetdoc.h"

class QDate;
class QLabel;
class QGridLayout;
class KDoubleNumInput;
class KBudgetEntryEditor;
class KLineEdit;
class KPushButton;

/**Displays and edits the budget values for an account
  *@author Richard Garand
  */

class KBudgetEditor : public QFrame
{
  Q_OBJECT
  public: 
    KBudgetEditor(QWidget* parent, KBudgetDoc* doc);
    virtual ~KBudgetEditor();

    /** selects a period */
    void selectPeriod(const KPeriod &period);
    /** selects an account by ID */
    void selectAccount(int id);

  public slots:
    /** changes the current month's budgeted value */
    void slotValueChanged(double value);
    /** slot to hide the panel */
    void slotHide();
    /** called when an account is modified */
    void slotAccountChanged(int acct, KBudgetDoc::ChangeAction action);
    
  signals:
    /** indicates that the account has been modified, passes the account ID */
    void valueChanged(int account);

  private:
    KBudgetDoc* m_doc;
    KPeriod m_period;
    int account;

    /* widgets */
    QLabel* prevMonthLabel;
    QLabel* prevMonthValue;
    QLabel* thisMonthLabel;
    KDoubleNumInput* thisMonthValue;
    QLabel* total;

/*    QPtrList<KBudgetEntryEditor> editors;
    QGridLayout *layout;
    int nextRow;*/

    /** if true, ignore valueChanged signals */
    bool m_selecting;
};

/* A class representing an entry in the budget editor; for internal use only. 
class KBudgetEntryEditor : public QObject
{
  Q_OBJECT
  public:
    / ** Creates a new budget entry editor as a child of p in l, using the given
      * description text and optionally making the description editable. Requires
      * that the layout already have the top and bottom rows. 
    KBudgetEntryEditor(KBudgetEditor *parent, QString text, bool editable = false);

    QString text() const;
    void setText(QString text);

    float value() const;
    void setValue(float amt);

  signals:
    void amountChanged(float value);
    void textChanged(QString text);

  public slots:
    void slotRemove();
    void slotRecurrence();

  private:
    / ** Moves the editor up or down by the given number of rows. 
    void move(int rows);

    KBudgetEditor *m_parent;

    QLabel *m_label;
    KLineEdit *m_lineEdit;
    KDoubleNumInput *m_amtEditor;
    QLabel *m_amtLabel;
    KPushButton *m_removeButton;
    KPushButton *m_recurrenceButton;
    
    QGridLayout *layout;

    // list of widgets by column (ptrs can be null)
    QValueVector<QWidget*> widgets;

    int row;
};*/

#endif
