/***************************************************************************
                          kbudgetview.h  -  description
                             -------------------
    begin                : Sat Aug 10 16:03:35 CST 2002
    copyright            : (C) 2002 by Richard Garand
    email                : richard@garandnet.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KBUDGETVIEW_H
#define KBUDGETVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

// include files for Qt
#include <qwidget.h>
#include <qvbox.h>
#include <qdatetime.h>
#include <qlistview.h>
#include <qmap.h>
#include <qvaluevector.h>

#include "account.h"
#include "kperiodselector.h"
#include "kbudgeteditor.h"
#include "kbudgetdoc.h"

class KBudgetViewItem;

/** The KBudgetView class provides the view widget for the KBudgetApp instance.	
 * The View instance inherits QWidget as a base class and represents the view object of a KTMainWindow. As KBudgetView is part of the
 * docuement-view model, it needs a reference to the document object connected with it by the KBudgetApp class to manipulate and display
 * the document structure provided by the KBudgetDoc class.
 * 	
 * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
 * @version KDevelop version 0.4 code generation
 */
class KBudgetView : public QVBox
{
  Q_OBJECT
  public:
    /** Constructor for the main view */
    KBudgetView(QWidget *parent = 0, const char *name=0);
    /** Destructor for the main view */
    ~KBudgetView();

    /** sets the display to the month and year specified */
    void selectMonth(int year, int month);

    /** returns a pointer to the document connected to the view instance. Mind that this method requires a KBudgetApp instance as a parent
     * widget to get to the window document pointer by calling the KBudgetApp::getDocument() method.
     *
     * @see KBudgetApp#getDocument
     */
    KBudgetDoc *getDocument() const;

    /** contains the implementation for printing functionality */
    void print(QPrinter *pPrinter);

    /** returns the id of the currently selected account or -1 */
    int currentAccount() const;
    
    /** Returns the period currently being displayed. */
    KPeriod curPeriod() const {
      return period->period();
    }

  public slots:
    /** changes the selected period */
    void slotPeriodChanged(KPeriod to);
    /** called when the document is changed */
    void slotDocumentChanged();
    /** called when an account is modified */
    void slotAccountChanged(int acct, KBudgetDoc::ChangeAction);
    /** called when the table selection is changed */
    void slotSelectionChanged(QListViewItem* item);
    /** opens a context menu */
    void slotContextMenuRequested(QListViewItem* item, const QPoint& pos);
    /** show transactiosn for the currently selected account */
    void slotTransactions();
    /** show transactiosn for the currently selected period */
    void slotAllTransactions();
    /** opens a dialog box to edit the currently selected account */
    void slotEditAccount();
    /** opens a new account dialog box */
    void slotNewAccount();
    /** opens a new transaction dialog box */
    void slotNewTransaction();
    /** reconciles the currently selected account */
    void slotReconcileAccount();
    /** select the given item */
    void slotSetSelected(QListViewItem* i);

  signals:
    /** set the enabled status for actions that require that an account be
      * selected */
    void accountSelectionStatus(bool);
    /** request a context menu */
    void contextMenuRequested(const QPoint& point);

  private:
    typedef QValueVector<KBudgetViewItem*> ViewItemVector;
    typedef QValueVector<AccountAccessor> AccessorVector;
  
    /** creates the summary display */
    void createSummary(AccessorVector accounts[3]);
    /** updates the expected change and expected balance on the statusbar */
    void updateStatusBar();
    /** adds account in section type with position pos, inserting it after previous in the main view */
    void addAccount(int type, int pos, Account& account, KBudgetViewItem* previous);

    KPeriodSelector* period;
    QListView* lv;
    KBudgetEditor* editor;

    /** totals rows for the sections */
    KBudgetViewItem* totals[3];
    /** header rows for the sections (insert accounts after these */
    KBudgetViewItem* headers[3];

    /** listview items for the accounts */
    ViewItemVector itemlist[3];
    /** map of items by account ID */
    QMap<int,KBudgetViewItem*> items;
    /** currently selected account */
    int selAccount;
};

#endif // KBUDGETVIEW_H
