/***************************************************************************
                          kbundoaction.h  -  description
                             -------------------
    begin                : Tue Sep 10 2002
    copyright            : (C) 2002 by Richard Garand
    email                : richard@garandnet.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KBUNDOACTION_H
#define KBUNDOACTION_H

#include <qstring.h>
#include <qptrlist.h>

#include <libkcal/recurrence.h>

class KBudgetDoc;
class Account;
class Transaction;
class QDate;

/**An undo/redo action
  *@author Richard Garand
  */

class KBUndoAction {
  public: 
    KBUndoAction(KBudgetDoc* doc);
    virtual ~KBUndoAction();

    virtual void undo() = 0;
    virtual void redo() = 0;
  protected:
    KBudgetDoc* m_doc;
};

class KBUndoGroup : public KBUndoAction {
  public:
    KBUndoGroup(KBudgetDoc* doc);
    ~KBUndoGroup();

    void undo();
    void redo();

    void add(KBUndoAction* act);
  private:
    QPtrList<KBUndoAction> list;
};

class KBAccountUndo : public KBUndoAction {
  public:
    enum actions { UA_NONE = 0, UA_ADD, UA_DEL, UA_RENAME, UA_STBALANCE, UA_BUDGET };

    KBAccountUndo(KBudgetDoc* doc, actions action, int id, QDate date = QDate());
    ~KBAccountUndo();

    void undo();
    void redo();
  private:
    void allocatePtr();
    void storeAccount();
    void recreateAccount();
  
    actions m_action;
    int m_id;

    Account* savedAcct;
    QString savedName;
    float savedValue;
    QDate savedDate;
};

class KBTransactionUndo : public KBUndoAction {
  public:
    enum actions { UT_NONE = 0, UT_ADD, UT_DEL, UT_DESCRIPTION, UT_MODIFIED };

    KBTransactionUndo(KBudgetDoc* doc, actions action, int id);
    ~KBTransactionUndo();

    void undo();
    void redo();
  private:
    void allocatePtr();
    void store();
    void recreate();

    actions m_action;
    int m_id;

    Transaction* saved;
    int savedFrom, savedTo;
    QString savedDesc;
    float savedValue;
    QDate savedDate;
    KCal::Recurrence savedRecurrence;
};

#endif
