/***************************************************************************
                          knewtransaction.cpp  -  description
                             -------------------
    begin                : Fri Aug 30 2002
    copyright            : (C) 2002 by Richard Garand
    email                : richard@garandnet.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qlayout.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qpushbutton.h>

#include <klocale.h>
#include <kmessagebox.h>
#include <kmainwindow.h>
#include <kdebug.h>

#include <libkdepim/kdateedit.h>

#include "knewtransaction.h"
#include "kbudgetdoc.h"
#include "koeditorrecurrence.h"

#include "knewtransaction.moc"

KTransactionDlg::KTransactionDlg(KMainWindow* parent, KBudgetDoc* doc, int tx, int currentAccount, QString button)
  : KDialogBase(Tabbed, "", Ok|Cancel|(button.isEmpty()?Ok:User1), Ok, parent, "knewtransaction", false),
  mainWin(parent), document(doc), id(tx)
{
  QWidget *page = addPage(i18n("Transaction"));
  trpage = page;
  grid = new QGridLayout(page);

  QLabel* lb = new QLabel(i18n("Date:"), page);
  grid->addWidget(lb, 0, 0);
  date = new KDateEdit(page);
  grid->addWidget(date, 0, 1);

  lb = new QLabel(i18n("From:"), page);
  grid->addWidget(lb, 1, 0);
  from = new KComboBox(page);
  grid->addWidget(from, 1, 1);
  from->setFocus();

  lb = new QLabel(i18n("To:"), page);
  grid->addWidget(lb, 2, 0);
  to = new KComboBox(page);
  grid->addWidget(to, 2, 1);

  lb = new QLabel(i18n("Amount:"), page);
  grid->addWidget(lb, 3, 0);
  value = new KDoubleNumInput(page);
  grid->addWidget(value, 3, 1);

  lb = new QLabel(i18n("Description:"), page);
  grid->addWidget(lb, 4, 0);
  description = new KLineEdit(page);
  grid->addWidget(description, 4, 1);

  // fill up extra space
  extraText = new QLabel("", page);
  // let's set the ALIGNMENT to WORD WRAP in case this is used to display a
  // message...
  extraText->setAlignment(Qt::WordBreak);
  grid->addMultiCellWidget(extraText, 5, 5, 0, 1);

  connect(date, SIGNAL(dateChanged(QDate)), SLOT(slotDateChanged(QDate)));

  value->setPrecision(2);
  value->setRange(-9999999, 9999999, 0.01, false);
  value->setFocus();

  page = addPage(i18n("Recurrence"));
  QBoxLayout *topLayout = new QVBoxLayout(page);
  recurrence = new KOEditorRecurrence(page);
  recurrence->setEnabled(false);
  topLayout->addWidget(recurrence);
  
  if ( !button.isEmpty() ) {
    setButtonText(User1, button);
  }
  
  int defaultFrom, defaultTo;
  if ( tx >= 0 ) {
    Transaction &tr = doc->getTransaction(id);
    date->setDate(tr.date());
    value->setValue(tr.value());
    description->setText(tr.description());
    recurrence->readTransaction(&tr);
    defaultFrom = tr.from();
    defaultTo = tr.to();
    id = tx;
  } else {
    recurrence->setDefaults(date->date(), date->date(), true);
    defaultFrom = doc->suggestFrom(currentAccount);
    defaultTo = doc->suggestTo(currentAccount);
  }

  accounts = doc->getAllAccounts();

  for ( unsigned int i = 0; i < accounts.count(); i++ ) {
    from->insertItem(accounts.at(i)->name());
    if ( accounts.current()->id() == defaultFrom )
      from->setCurrentItem(i);
    to->insertItem(accounts.current()->name());
    if ( accounts.current()->id() == defaultTo )
      to->setCurrentItem(i);
  }
}

KTransactionDlg::~KTransactionDlg()
{
}

void KTransactionDlg::slotDateChanged(QDate date)
{
  recurrence->setDateTimes(date, date);
}

KNewTransaction::KNewTransaction(KMainWindow* parent, KBudgetDoc* doc, int currentAccount, int tx)
  : KTransactionDlg(parent, doc, tx, currentAccount, "")
{
  if ( tx >= 0 )
    setCaption(i18n("Edit Transaction"));
  else
    setCaption(i18n("New Transaction"));
}

KNewTransaction::~KNewTransaction()
{
}

void KNewTransaction::slotOk()
{
  QDate when = date->date();
  float amount = value->value();
  QString desc = description->text();
  int fromAcct = accounts.at(from->currentItem())->id();
  int toAcct = accounts.at(to->currentItem())->id();

  // new transaction...
  if ( id < 0 ) {
    if ( fromAcct == toAcct )
      KMessageBox::sorry(0, i18n("The from account and to account must be different."));
    else {
      int id = document->newTransaction(fromAcct, toAcct, amount, when, desc);
      // this is safe for undo, because the new transaction action only needs
      // to see the data when we actually undo it
      recurrence->writeTransaction(&document->getTransaction(id));
      KDialogBase::slotOk();
    }
  } else {
    // or modify existing
    Transaction& tr = document->getTransaction(id);
    document->beginUndoGroup();
    tr.setData(fromAcct, toAcct, amount, when, desc);
    recurrence->writeTransaction(&tr);
    document->endUndoGroup();
    document->setModified(true);
    KDialogBase::slotOk();
  }
}

KNewRecurrence::KNewRecurrence(KMainWindow *parent, KBudgetDoc *doc, int transaction)
  : KTransactionDlg(parent, doc, transaction, -1, i18n("Later"))
{
  setCaption(i18n("New Transaction"));
  date->setDate(QDate::currentDate());
  extraText->setText(i18n("This is a recurring transaction. You can click Ok to create the new transaction, Cancel to skip this recurrence, or Later to be reminded later."));

  QLabel *lb = new QLabel(i18n("Remind in:"), trpage);
  grid->addWidget(lb, 6, 0);
  QHBox *hb = new QHBox(trpage);
  grid->addWidget(hb, 6, 1);
  count = new KIntNumInput(hb);
  units = new KComboBox(hb);
  units->insertItem(i18n("minutes"));
  units->insertItem(i18n("hours"));
  units->insertItem(i18n("days"));
  units->insertItem(i18n("weeks"));
  units->insertItem(i18n("months"));

  // default to 1 day... what is a reasonable default?
  count->setValue(1);
  units->setCurrentItem(2);
}

KNewRecurrence::~KNewRecurrence()
{
}

void KNewRecurrence::slotOk()
{
  QDate when = date->date();
  float amount = value->value();
  QString desc = description->text();
  int fromAcct = accounts.at(from->currentItem())->id();
  int toAcct = accounts.at(to->currentItem())->id();
  
  if ( fromAcct == toAcct ) {
    KMessageBox::sorry(0, i18n("The from account and to account must be different."));
    return;
  }

  // create new transaction
  // TODO: make sure the recurrence changes are included in the undo
  document->beginUndoGroup();
  int nid = document->newTransaction(fromAcct, toAcct, amount, when, desc);
  document->getTransaction(nid).recurrence() = document->getTransaction(id).recurrence();
  document->getTransaction(id).recurrence().unsetRecurs();

  document->getTransaction(id).recurrenceUpdated();
  document->getTransaction(nid).recurrenceUpdated();
  document->endUndoGroup();

  KDialogBase::slotOk();
}

void KNewRecurrence::slotCancel()
{
  // the user doesn't want to instantiate this recurrence, so we add it as an
  // exception using the original next recurrence date
  Transaction &tr = document->getTransaction(id);
  QValueList<QDate> exceptions = tr.recurrence().exDates();
  exceptions.push_back(tr.nextRecurrence());
  tr.recurrence().setExDates(exceptions);
  tr.recurrenceUpdated();
  
  KDialogBase::slotCancel();
}

void KNewRecurrence::slotUser1()
{
  // this means wait a little before showing the recurrence again
  QDateTime next = QDateTime::currentDateTime();
  int c = count->value();
  switch ( units->currentItem() ) {
  case 0: next = next.addSecs(60*c); break;
  case 1: next = next.addSecs(3600*c); break;
  case 2: next = next.addDays(c); break;
  case 3: next = next.addDays(7*c); break;
  case 4: next = next.addMonths(c); break;
  default: break;
  }
  kdDebug() << "c:" << c << "; u:" << units->currentItem() << "; from " << QDateTime::currentDateTime().toString(Qt::ISODate) << " to " << next.toString(Qt::ISODate) << endl;
  document->getTransaction(id).setNextNotify(next);

  KDialogBase::slotCancel();
}
