/***************************************************************************
                          knewtransaction.h  -  description
                             -------------------
    begin                : Fri Aug 30 2002
    copyright            : (C) 2002 by Richard Garand
    email                : richard@garandnet.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KNEWTRANSACTION_H
#define KNEWTRANSACTION_H

#include <kdialogbase.h>
#include <kdatepicker.h>
#include <kcombobox.h>
#include <klineedit.h>
#include <qvaluelist.h>
#include <knuminput.h>

#include "transaction.h"

class KMainWindow;
class KBudgetDoc;
class Account;
class KDateEdit;
class KOEditorRecurrence;
class QLabel;
class QGridLayout;

/** A base dialog for creating or editing a transaction. */
class KTransactionDlg : public KDialogBase
{
  Q_OBJECT
  public:
    /** Creates a generic transaction editing dialog, loading the data for the
      * given transaction id if >= 0 and setting the from/to accounts based
      * on currentAccount if not. If button is not empty, a third button will
      * be created with the contents of button as its text. */
    KTransactionDlg(KMainWindow* parent, KBudgetDoc* doc, int id, int currentAccount, QString button);
    ~KTransactionDlg();
    
  protected:
    const KMainWindow* mainWin;
    KBudgetDoc* document;
    int id;
    
    QWidget *trpage;
    QGridLayout *grid;

    KDateEdit* date;
    KComboBox* from;
    KComboBox* to;
    KDoubleNumInput* value;
    KLineEdit* description;
    QPtrList<Account> accounts;
    QLabel *extraText;
    
    KOEditorRecurrence *recurrence;
    
  private slots:
    void slotDateChanged(QDate date);
};

/**Creates and handles a new transaction dialog box. Can only be used in
  *"new transaction" mode when there are accounts.
  *@author Richard Garand
  */
class KNewTransaction : public KTransactionDlg  {
  Q_OBJECT
  public:
    /** creates the dialog box - current account can be set to a currently selected
      * account id to have the from and to combo boxes use sensible defaults, and
      * transaction can be set to a tid to make the dialog in "edit transaction"
      * mode (instead of "new transaction" mode) */
    KNewTransaction(KMainWindow* parent, KBudgetDoc* doc, int currentAccount = -1, int transaction = -1);
    ~KNewTransaction();

  private slots:
    void slotOk();
};

/** Creates a dialog box for instantiating a new recurrence from an existing
  * transaction. */
class KNewRecurrence : public KTransactionDlg
{
  Q_OBJECT
  public:
    KNewRecurrence(KMainWindow *parent, KBudgetDoc *doc, int transaction);
    ~KNewRecurrence();

  private slots:
    void slotOk();
    void slotCancel();
    void slotUser1();
    
  private:
    KIntNumInput *count;
    KComboBox *units;
};

#endif
