/***************************************************************************
                          kperiod.cpp  -  description
                             -------------------
    begin                : Sun Dec 22 2002
    copyright            : (C) 2002 by Richard Garand
    email                : richard@garandnet.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qdatetime.h>
#include <kglobal.h>
#include <klocale.h>

#include "kperiod.h"

KPeriod::KPeriod()
  : m_type(All)
{
}

KPeriod::KPeriod(int year, int month)
  : m_from(year, month, 1), m_type(Month)
{

}

KPeriod::KPeriod(QDate from, QDate to)
  : m_from(from), m_to(to), m_type(DateRange)
{

}

KPeriod::~KPeriod()
{
}

QString KPeriod::name() const
{
  if ( m_type == Month )
    return KGlobal::locale()->monthName(m_from.month());
  else if ( m_type == DateRange ) {
    KLocale* loc = KGlobal::locale();
    return QString("%1 - %2").arg(loc->formatDate(m_from, true)).arg(loc->formatDate(m_to, true));
  }
  return QString("");
}

void KPeriod::selectMonth(int year, int month)
{
  m_from.setYMD(year, month, 1);
  m_type = Month;
}

void KPeriod::addYears(int years)
{
  switch ( m_type ) {
  case DateRange:
    m_to.addYears(years);
  case Month:
    m_from.addYears(years);
  default:
    break;
  }
}

void KPeriod::addMonths(int months)
{
  switch ( m_type ) {
  case DateRange:
    m_to.addMonths(months);
  case Month:
    m_from.addMonths(months);
  default:
    break;
  }
}

void KPeriod::previous()
{
  if ( m_type == Month )
    m_from = m_from.addMonths(-1);
}

void KPeriod::next()
{
  if ( m_type == Month )
    m_from = m_from.addMonths(1);
}

int KPeriod::firstYear() const
{
  return m_from.year();
}

int KPeriod::firstMonth() const
{
  return m_from.month();
}

QDate KPeriod::firstDay() const
{
  return m_from;
}

int KPeriod::length() const
{
  switch ( m_type ) {
  case Month:
    return m_from.daysInMonth();
  case DateRange:
    return m_from.daysTo(m_to);
  default:
    return 0;
  }
}

bool KPeriod::contains(const QDate date) const
{
  switch ( m_type ) {
  case Month:
    return date.year() == m_from.year() && date.month() == m_from.month();
  case DateRange:
    return date >= m_from && date <= m_to;
  default:
    return true;
  }
}

bool KPeriod::containsYear(const int year) const
{
  switch ( m_type ) {
  case Month:
    return year == m_from.year();
  case DateRange:
    return year >= m_from.year() && year <= m_to.year();
  default:
    return true;
  }
}

bool KPeriod::containsMonth(const int year, const int month) const
{
  switch ( m_type ) {
  case Month:
    return year == m_from.year() && month == m_from.month();
  case DateRange:
    // return true if m_from <= (year, month) and m_to >= (year, month) (ignoring the day)
    return (m_from.year() < year || (m_from.year() == year && m_from.month() <= month)) \
      && (m_to.year() > year || (m_to.year() == year && m_from.month() >= month));
  default:
    return true;
  }
}

KPeriod::Types KPeriod::type() const
{
  return m_type;
}
