/***************************************************************************
                          kperiodselector.cpp  -  description
                             -------------------
    begin                : Sun Dec 22 2002
    copyright            : (C) 2002 by Richard Garand
    email                : richard@garandnet.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qlabel.h>
#include <qpushbutton.h>
#include <kcombobox.h>
#include <knuminput.h>
#include <kiconloader.h>
#include <qdatetime.h>
#include <kglobal.h>
#include <klocale.h>

#include "kperiodselector.h"
#include "kperiodselector.moc"

KPeriodSelector::KPeriodSelector(QWidget *parent, const char *name ) : QHBox(parent,name)
{
  prevButton = new QPushButton(this);
  QPixmap pix = SmallIcon("back");
  QIconSet set = QIconSet(pix);
  prevButton->setIconSet(set);

  QLabel* spacer1 = new QLabel(this);
  setStretchFactor(spacer1, 1);

  month = new KComboBox(this);
  for ( int i = 1; i <= 12; i++ )
    month->insertItem(KGlobal::locale()->monthName(i));

  year = new KIntNumInput(this);

  spacer2 = new QLabel(this);
  setStretchFactor(spacer2, 1);

  nextButton = new QPushButton(this);
  pix = SmallIcon("forward");
  set = QIconSet(pix);
  nextButton->setIconSet(set);

  year->setMinValue(1752);
  year->setMaxValue(8000);

  connect(prevButton, SIGNAL(clicked()), this, SLOT(slotPrevClick()));
  connect(month, SIGNAL(activated(int)), this, SLOT(slotMonthActivate(int)));
  connect(year, SIGNAL(valueChanged(int)), this, SLOT(slotYearChange(int)));
  connect(nextButton, SIGNAL(clicked()), this, SLOT(slotNextClick()));
}

KPeriodSelector::~KPeriodSelector()
{
}

void KPeriodSelector::setPeriod(KPeriod to)
{
  if ( to.type() == KPeriod::Month ) {
    m_period = to;
    changed();
  }
}

KPeriod KPeriodSelector::period() const
{
  return m_period;
}

void KPeriodSelector::slotPrevClick()
{
  m_period.previous();
  changed();
}

void KPeriodSelector::slotMonthActivate(int month)
{
  m_period.selectMonth(year->value(), month + 1);
  changed();
}

void KPeriodSelector::slotYearChange(int year)
{
  m_period.selectMonth(year, month->currentItem() + 1);
  changed();
}

void KPeriodSelector::slotNextClick()
{
  m_period.next();
  changed();
}

void KPeriodSelector::changed()
{
  int pmonth = m_period.firstMonth() - 1;
  if ( !pmonth ) pmonth = 12;
  QString mstr = KGlobal::locale()->monthName(pmonth);

  QString filler;
  prevButton->setText(mstr + filler.fill(' ', 10 - mstr.length()));

  int nmonth = m_period.firstMonth() + 1;
  if ( nmonth == 13 ) nmonth = 1;
  mstr = KGlobal::locale()->monthName(nmonth);

  nextButton->setText(filler.fill(' ', 10 - mstr.length()) + mstr);

  month->setCurrentItem(m_period.firstMonth() - 1);
  year->setValue(m_period.firstYear());

  emit periodChanged(m_period);
}
