/***************************************************************************
                          ktransactionlist.h  -  description
                             -------------------
    begin                : Sat Sep 7 2002
    copyright            : (C) 2002 by Richard Garand
    email                : richard@garandnet.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KTRANSACTIONLIST_H
#define KTRANSACTIONLIST_H

#include <qwidget.h>
#include <qmap.h>
#include <klistview.h>

#include "kbudgetdoc.h"

class QDate;
class KTransactionListItem;
class QPoint;
class KPeriod;
class KPeriodSelector;

/**A transaction listing window, with optional balance and checkbox columns.
  *When the default columns are used, the balance column will be shown
  *automatically if the account has the type Cash.
  *@author Richard Garand
  */

class KTransactionList : public KListView  {
  Q_OBJECT
  public:
    enum Profiles { NormalList, Reconcile };
    enum Columns { Date=0, Debits, Credits, Value, Balance, From, To, Description };

    KTransactionList(KBudgetDoc* doc, QWidget *parent, int account=-1, Profiles profile = NormalList);
    ~KTransactionList();

    /** returns true if the balance column and initial balance are shown */
    bool showBalance() const;
    /** returns true if all transactions for all accounts are being listed */
    bool showAllAccounts() const;

    /** returns the index of the given column */
    int getColumnIndex(Columns col) const;

    /** returns the id of the currently selected transaction or -1 */
    int getCurrent();

    /** returns a pointer to the document */
    KBudgetDoc* getDocument();

    /** clears the list */
    void clear();
    
    /** adds the given transaction to the list (doesn't work with balances) */
    void addTransaction(int id);

  public slots:
    /** display the given period */
    void slotPeriodChanged(KPeriod to);

    void slotDeleteTransaction();
    void slotTransactionChanged(int id, KBudgetDoc::ChangeAction action);
    void slotContextMenuRequested(QListViewItem*, const QPoint &p, int);
    void slotSelectionChanged();
    void slotCurrentChanged(QListViewItem* i);
    void slotEditTransaction();

  signals:
    /** passing on the QListView signal */
    void contextMenuRequested(const QPoint& p);
    /** indicates that actions for transactions should be enabled or disabled */
    void transactionFuncsEnabled(bool enabled);

    /** emitted when the ending balance for the current period changes */
    void balanceChanged(float balance);

  private:
    typedef QMap<int, KTransactionListItem*> ListItemMap;

    /** the document associated with this transaction list */
    KBudgetDoc* m_doc;
    /** the account whose transactions are being shown */
    int m_account;
    /** the currently selected period */
    KPeriod m_period;
    /** the current initial balance */
    float m_initialBalance;

    bool m_allAccounts;
    bool m_showBalance;

    int colIndices[8];

    /** total row */
    KTransactionListItem* total;
    /** first item in the list (initial balance if available) - this is assumed to be
      * valid after widget initialization */
    KTransactionListItem* first;
    /** listview items by transaction id */
    ListItemMap items;
};

#endif
