/***************************************************************************
                          ktransactionlistitem.cpp  -  description
                             -------------------
    begin                : Sun Sep 8 2002
    copyright            : (C) 2002 by Richard Garand
    email                : richard@garandnet.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qdatetime.h>

#include <kglobal.h>
#include <klocale.h>
#include <kdebug.h>

#include "ktransactionlistitem.h"
#include "transaction.h"

KTransactionListItem::KTransactionListItem(KTransactionList* parent)
  : KListViewItem(parent), m_transaction(0), m_debits(0), m_credits(0),
  m_priority(1), m_account(-1), m_prev(0), m_next(0)
{
  setSelectable(false);
}

KTransactionListItem::KTransactionListItem(KTransactionList* parent, float debits, float credits)
  : KListViewItem(parent), m_transaction(0), m_debits(debits), m_credits(credits),
  m_priority(2), m_account(-1), m_prev(0), m_next(0)
{
  setText(column(KTransactionList::Date), i18n("Total"));
  if ( !debits && !credits )
    setText(column(KTransactionList::Description), i18n("<no transactions>"));
  else
    setText(column(KTransactionList::Description), "");

  setSelectable(false);
}

KTransactionListItem::KTransactionListItem(KTransactionList* parent, Transaction* t, float balance, int account,\
    KTransactionListItem* total)
  : KListViewItem(parent), m_transaction(t), m_debits(0), m_credits(0),
  m_balance(balance), m_date(t->date()), m_priority(0), m_account(account),
  m_prev(0), m_next(0), m_total(total)
{
  setText(column(KTransactionList::Date), KGlobal::locale()->formatDate(m_date, true));

  if ( parent->showAllAccounts() ) {
    m_debits = t->value();
    setText(column(KTransactionList::Value), KGlobal::locale()->formatMoney(m_debits));
  } else if ( t->from() == account ) {
    m_debits = t->value();
    setText(column(KTransactionList::Debits), KGlobal::locale()->formatMoney(m_debits));
  } else if ( t->to() == account ) {
    m_credits = t->value();
    setText(column(KTransactionList::Credits), KGlobal::locale()->formatMoney(m_credits));
  }

  if ( parent->showBalance() )
    setText(column(KTransactionList::Balance), KGlobal::locale()->formatMoney(m_balance));

  if ( parent->showAllAccounts() ) {
    setText(column(KTransactionList::From), parent->getDocument()->getAccount(t->from()).name());
    setText(column(KTransactionList::To), parent->getDocument()->getAccount(t->to()).name());
  }

  setText(column(KTransactionList::Description), t->description());

  if ( m_total )
    m_total->add(m_debits, m_credits);

  setRenameEnabled(column(KTransactionList::Description), true);
}

KTransactionListItem::KTransactionListItem(KTransactionList* parent, QDate date, float balance)
  : KListViewItem(parent), m_transaction(0), m_debits(0), m_credits(0),
  m_balance(balance), m_date(date), m_priority(0), m_account(-1), m_prev(0),
  m_next(0)
{
  setText(column(KTransactionList::Date), KGlobal::locale()->formatDate(m_date, true));
  setText(column(KTransactionList::Balance), KGlobal::locale()->formatMoney(m_balance));
  setText(column(KTransactionList::Description), i18n("Initial balance"));

  setSelectable(false);
}

KTransactionListItem::~KTransactionListItem()
{
  // update balances
  if ( m_next )
    m_next->add(m_debits - m_credits);

  // update total
  if ( m_transaction && m_total )
    m_total->add(-m_debits, -m_credits);

  // unlink
  if ( m_prev )
    m_prev->m_next = m_next;
  if ( m_next )
    m_next->m_prev = m_prev;
}

void KTransactionListItem::insertAfter(KTransactionListItem* item)
{
  // link up the items
  m_next = item->m_next;
  m_prev = item;
  item->m_next = this;
  if ( m_next )
    m_next->m_prev = this;

  // initialize the balance
  m_balance = item->m_balance;

  // update the balance and propagate the change
  add(m_credits - m_debits);
}

void KTransactionListItem::setNext(KTransactionListItem* next)
{
  m_next = next;
  if ( m_next )
    m_next->m_prev = this;
}

void KTransactionListItem::reread()
{
  if ( !m_transaction )
    return;

  m_date = m_transaction->date();
  setText(column(KTransactionList::Date), KGlobal::locale()->formatDate(m_date, true));

  float debits=0, credits=0;
  if ( m_transaction->from() == m_account ) {
    debits = m_transaction->value();
    setText(column(KTransactionList::Debits), KGlobal::locale()->formatMoney(debits));
  } else if ( m_transaction->to() == m_account ) {
    credits = m_transaction->value();
    setText(column(KTransactionList::Credits), KGlobal::locale()->formatMoney(credits));
  }
  setText(column(KTransactionList::Description), m_transaction->description());

  add(credits - m_credits - debits + m_debits);

  if ( m_total )
    m_total->add(debits - m_debits, credits - m_credits);
    
  // check if the item needs to be moved up (less than previous); this could be
  // done without relinking at each step, but the code is uglier
  while ( m_prev && compare(m_prev, 0, true) == -1 ) {
    // delink
    m_prev->m_next = m_next;
    if ( m_next ) m_next->m_prev = m_prev;

    // update balances
    m_prev->localAdd(m_credits - m_debits);
    localAdd(m_prev->m_debits - m_prev->m_credits);
    
    // relink before m_prev
    m_next = m_prev;
    m_prev = m_prev->m_prev;
    if ( m_prev ) m_prev->m_next = this;
    m_next->m_prev = this;
  }
  // check if the item needs to be moved down (greater than next)
  while ( m_next && compare(m_next, 0, true) == 1 ) {
    // delink
    if ( m_prev ) m_prev->m_next = m_next;
    m_next->m_prev = m_prev;

    // update balance
    m_next->localAdd(m_debits - m_credits);
    localAdd(m_next->m_credits - m_next->m_debits);
    
    // relink after m_next
    m_prev = m_next;
    m_next = m_next->m_next;
    m_prev->m_next = this;
    if ( m_next ) m_next->m_prev = this;
  }

  m_debits = debits;
  m_credits = credits;
  
  listView()->sort();
}

KTransactionListItem* KTransactionListItem::next() const
{
  return m_next;
}

int KTransactionListItem::compare(QListViewItem* i, int col, bool asc) const
{
  // we have to assume this will work
  KTransactionListItem* other = (KTransactionListItem*)i;

  if ( m_priority < other->m_priority )
    return asc ? -1 : 1;
  else if ( m_priority > other->m_priority )
    return asc ? 1 : -1;

  if ( col == column(KTransactionList::Date) ) {
    if ( m_date > other->m_date )
      return 1;
    else if ( m_date < other->m_date )
      return -1;
  } else if ( col == column(KTransactionList::Debits) ) {
    if ( m_debits > other->m_debits )
      return 1;
    else if ( m_debits < other->m_debits )
      return -1;
  } else if ( col == column(KTransactionList::Credits) ) {
    if ( m_credits > other->m_credits )
      return 1;
    else if ( m_credits < other->m_credits )
      return -1;
  } else if ( col == column(KTransactionList::Balance) ) {
    if ( m_balance > other->m_balance )
      return 1;
    else if ( m_balance < other->m_balance )
      return -1;
  } else if ( col == column(KTransactionList::Description) ) {
    if ( text(3) > other->text(3) )
      return 1;
    else if ( text(3) < other->text(3) )
      return -1;
  }

  if ( tid() > other->tid() )
    return 1;
  else if ( tid() < other->tid() )
    return -1;

  kdDebug() << "Warning: KTransactionListItem::compare() returning 0" << endl;
  return 0;
}

int KTransactionListItem::tid() const
{
  if ( m_transaction )
    return m_transaction->id();
  else
    return -1;
}

float KTransactionListItem::balanceChange() const
{
  if ( !m_transaction )
    return 0;

  if ( m_transaction->from() == m_account )
    return -m_transaction->value();
  else
    return m_transaction->value();
}

void KTransactionListItem::okRename(int col)
{
  QListViewItem::okRename(col);
  if ( col == column(KTransactionList::Description) )
    m_transaction->setDescription(text(3));
}

void KTransactionListItem::add(float balance)
{
  for ( KTransactionListItem* item = this; item; item = item->m_next ) {
    item->m_balance += balance;
    item->setText(column(KTransactionList::Balance), KGlobal::locale()->formatMoney(item->m_balance));
  }
}

void KTransactionListItem::localAdd(float balance)
{
  m_balance += balance;
  setText(column(KTransactionList::Balance), KGlobal::locale()->formatMoney(m_balance));
}

void KTransactionListItem::add(float debits, float credits)
{
  m_debits += debits;
  m_credits += credits;

  setText(column(KTransactionList::Debits), KGlobal::locale()->formatMoney(m_debits));
  setText(column(KTransactionList::Credits), KGlobal::locale()->formatMoney(m_credits));
  setText(column(KTransactionList::Description), "");
}

int KTransactionListItem::column(KTransactionList::Columns col) const
{
  return ((KTransactionList*)listView())->getColumnIndex(col);
}
