/***************************************************************************
                          ktransactionlistitem.h  -  description
                             -------------------
    begin                : Sun Sep 8 2002
    copyright            : (C) 2002 by Richard Garand
    email                : richard@garandnet.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KTRANSACTIONLISTITEM_H
#define KTRANSACTIONLISTITEM_H

#include <klistview.h>

#include "ktransactionlist.h"

class Transaction;
class QDate;

/**A transaction list view item
  *@author Richard Garand
  */

class KTransactionListItem : public KListViewItem  {
  public:
    /** default contrustor; creates a blank non-selectable row */
    KTransactionListItem(KTransactionList* parent);
    /** creates a non-selectable total row with the given values */
    KTransactionListItem(KTransactionList* parent, float debits, float credits);
    /** creates a transaction row (t must be a valid pointer) */
    KTransactionListItem(KTransactionList* parent, Transaction* t, float balance, \
      int account, KTransactionListItem* total);
    /** create a non-selectable initial balance row */
    KTransactionListItem(KTransactionList* parent, QDate date, float balance);
    ~KTransactionListItem();

    /** inserts this item after the given item in the list */
    void insertAfter(KTransactionListItem* item);
    /** sets the next item */
    void setNext(KTransactionListItem* next);

    /** rereads data for the transaction (only works if the transaction
      * is set) */
    void reread();

    /** returns a pointer to the next item in default order (ascending
      * by date, id) */
    KTransactionListItem* next() const;

    /** compare to another KTranasctionListItem */
    int compare(QListViewItem* i, int col, bool asc) const;

    /** return the transaction ID or -1 */
    int tid() const;

    /** returns the value to add to the balance for this item */
    float balanceChange() const;

    /** add amount to m_balance and call m_next->add(amount) */
    void add(float balance);
  private:
    /** overrides QListView::okRename() to set description */
    void okRename(int col);
    /** adds the given amounts to m_{debits,credits} and sets the text for
      * both columns; clears the description column */
    void add(float debits, float credits);
    /** Adds the given amount to the balance; does not do the same for m_next. */
    void localAdd(float balance);
    /** returns the index of the given column */
    int column(KTransactionList::Columns col) const;
  
    Transaction* m_transaction;
    float m_debits, m_credits, m_balance;
    QDate m_date;
    int m_priority, m_account;
    KTransactionListItem* m_prev;
    KTransactionListItem* m_next;
    /** the total row - may not be valid when !m_transaction */
    KTransactionListItem* m_total;
};

#endif
