/***************************************************************************
                          ktransactionwindow.cpp  -  description
                             -------------------
    begin                : Sat Sep 7 2002
    copyright            : (C) 2002 by Richard Garand
    email                : richard@garandnet.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kstdaction.h>
#include <kaction.h>
#include <klocale.h>
#include <kconfig.h>
#include <kglobal.h>
#include <kkeydialog.h>
#include <kedittoolbar.h>
#include <qdatetime.h>
#include <qpopupmenu.h>
#include <qvbox.h>

#include "ktransactionwindow.h"
#include "ktransactionlist.h"
#include "kbudgetdoc.h"
#include "kperiodselector.h"
#include "knewtransaction.h"

QPtrList<KTransactionWindow> KTransactionWindow::instances;

KTransactionWindow::KTransactionWindow(KBudgetDoc* doc, KPeriod period, int account)
  : KMainWindow(0, "transaction_window")
{
  QVBox* vb = new QVBox(this);

  m_period = new KPeriodSelector(vb);
  
  transactionWidget = new KTransactionList(doc, vb, account);
  setCentralWidget(vb);

  instances.append(this);

  (KStdAction::close(this, SLOT(close()), actionCollection()))->setStatusText(i18n("Closes the window"));
  transactionNew = new KAction(i18n("&New"), "mail_forward", Qt::Key_T, this, SLOT(slotNewTransaction()), actionCollection(), "transaction_new");
  transactionEdit = new KAction(i18n("&Edit"), "edit", Qt::Key_E, transactionWidget, SLOT(slotEditTransaction()), actionCollection(), "transaction_edit");
  transactionDelete = new KAction(i18n("&Delete"), "editdelete", Qt::Key_D, transactionWidget, SLOT(slotDeleteTransaction()), actionCollection(), "transaction_delete");
  showToolbar = KStdAction::showToolbar(this, SLOT(slotToggleToolBar()), actionCollection());
  KStdAction::keyBindings(this, SLOT(slotConfKeys()), actionCollection());
  KStdAction::configureToolbars(this, SLOT(slotConfToolbar()), actionCollection());

  slotTransactionFuncsEnabled(false);

  transactionEdit->setStatusText(i18n("Edits the selected transaction"));
  transactionDelete->setStatusText(i18n("Deletes the selected transaction"));

  initGUI();

  KConfig *conf = KGlobal::config();
  conf->setGroup("transactionWindow_options");
  resize(400,300);    // using random numbers ripped from KNode :)
  applyMainWindowSettings(conf);
  showToolbar->setChecked(!toolBar()->isHidden());

  mainPopup = static_cast<QPopupMenu*>(factory()->container("popup", this));
  if (!mainPopup) mainPopup = new QPopupMenu(this);

  connect(doc, SIGNAL(documentChanged()), SLOT(close()));
  connect(doc, SIGNAL(accountChanged(int, KBudgetDoc::ChangeAction)), SLOT(slotAccountChanged(int, KBudgetDoc::ChangeAction)));
  connect(m_period, SIGNAL(periodChanged(KPeriod)), transactionWidget, SLOT(slotPeriodChanged(KPeriod)));
  connect(m_period, SIGNAL(periodChanged(KPeriod)), SLOT(slotPeriodChanged()));
  connect(transactionWidget, SIGNAL(contextMenuRequested(const QPoint&)), SLOT(slotContextMenuRequested(const QPoint&)));
  connect(transactionWidget, SIGNAL(transactionFuncsEnabled(bool)), SLOT(slotTransactionFuncsEnabled(bool)));

  m_account = account;
  m_doc = doc;
  m_period->setPeriod(period);
}

KTransactionWindow::~KTransactionWindow()
{
  instances.removeRef(this);
  KConfig *conf = KGlobal::config();
  conf->setGroup("transactionWindow_options");
  saveMainWindowSettings(conf);
}

bool KTransactionWindow::closeAllWindows()
{
  // TODO: limit this to the window that created this one
  QPtrList<KTransactionWindow> list = instances;
  for (KTransactionWindow* w = list.first(); w; w = list.next())
    w->close();
  return true;
}

void KTransactionWindow::slotToggleToolBar()
{
  if(toolBar()->isVisible())
    toolBar()->hide();
  else
    toolBar()->show();
}

void KTransactionWindow::slotConfKeys()
{
  KKeyDialog::configure(actionCollection());
}

void KTransactionWindow::slotConfToolbar()
{
  KEditToolbar *dlg = new KEditToolbar(guiFactory(),this);
  if (dlg->exec())
    initGUI();
  delete dlg;
}

void KTransactionWindow::slotContextMenuRequested(const QPoint& p)
{
  mainPopup->popup(p);
}

void KTransactionWindow::slotTransactionFuncsEnabled(bool enabled)
{
  transactionEdit->setEnabled(enabled);
  transactionDelete->setEnabled(enabled);
}

void KTransactionWindow::slotNewTransaction()
{
  (new KNewTransaction(this, m_doc, m_account))->show();
}

void KTransactionWindow::slotPeriodChanged()
{
  makeCaption();
}

void KTransactionWindow::slotAccountChanged(int id, KBudgetDoc::ChangeAction action)
{
  if ( id == m_account && action == KBudgetDoc::Modified )
    makeCaption();
}

void KTransactionWindow::initGUI()
{
  createGUI("ktransactionwindowui.rc");
}

void KTransactionWindow::makeCaption()
{
  QString pname = m_period->period().name();
  if ( m_account == -1 )
    setCaption(i18n("The argument will be a month or period name", "Transactions for %1").arg(pname));
  else
    setCaption(i18n("The arguments will be a month or period name and an account name", \
      "Transactions for %1 (%2)").arg(pname).arg(m_doc->getAccount(m_account).name()));
}

#include "ktransactionwindow.moc"
